package eu.mhsl.craftattack.spawn.common.appliances.internal.debug.command;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.common.appliances.internal.debug.Debug;
import eu.mhsl.craftattack.spawn.core.util.text.ComponentUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class AppliancesCommand extends ApplianceCommand<Debug> {
    public AppliancesCommand() {
        super("appliances");
    }

    @Override
    protected void execute(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        ComponentBuilder<TextComponent, TextComponent.Builder> componentBuilder = Component.text()
            .append(Component.text(Main.instance().getAppliances().size()))
            .append(Component.text(" appliances running:"))
            .appendNewline();

        Main.instance().getAppliances().forEach(appliance -> {
            List<ApplianceCommand<?>> commands = appliance.getCommands();
            List<Listener> listener = appliance.getListeners();

            componentBuilder
                .append(Component.text(appliance.getClass().getSimpleName(), NamedTextColor.GREEN)
                    .hoverEvent(HoverEvent.showText(Component.text(appliance.getClass().getName()))))
                .append(Component.text(": ", NamedTextColor.DARK_GRAY))
                .append(Component.text(commands.size() + " Commands", NamedTextColor.GRAY)
                    .hoverEvent(HoverEvent.showText(commands.stream()
                        .map(applianceCommand -> Component.text()
                            .append(Component.text(applianceCommand.commandName, NamedTextColor.DARK_GREEN))
                            .append(Component.text(": "))
                            .append(Component.text(applianceCommand.getClass().getName()))
                            .build())
                        .reduce(ComponentUtil::appendWithNewline)
                        .orElse(Component.text("No commands available")))))
                .append(Component.text(", ", NamedTextColor.GRAY))
                .append(Component.text(listener.size() + " Listener", NamedTextColor.GRAY)
                    .hoverEvent(HoverEvent.showText(listener.stream()
                        .map(eventHandler -> Component.text()
                            .append(Component.text(eventHandler.getClass().getSimpleName(), NamedTextColor.DARK_GREEN))
                            .append(Component.text(": "))
                            .append(Component.text(eventHandler.getClass().getName()))
                            .build())
                        .reduce(ComponentUtil::appendWithNewline)
                        .orElse(Component.text("No listeners available")))))
                .appendNewline();
        });

        componentBuilder.append(Component.text(Main.instance().getClass().getName(), NamedTextColor.GRAY));

        sender.sendMessage(componentBuilder.build());
    }
}
