package eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.displayName;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.util.server.Floodgate;
import eu.mhsl.craftattack.spawn.core.util.text.ComponentUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Objects;
import java.util.logging.Level;

public class DisplayName extends Appliance {
    public interface Prefixed {
        @Nullable
        Component getNamePrefix(Player player);
    }

    public interface Colored {
        @Nullable
        TextColor getNameColor(Player player);
    }

    public void update(Player player) {
        List<Colored> coloring = Main.instance().getAppliances().stream()
            .filter(appliance -> appliance instanceof Colored)
            .map(appliance -> (Colored) appliance)
            .toList();

        if(coloring.size() > 1) throw new IllegalStateException(
            "There are two or more appliances which provide coloring for player names. This is currently not supported!"
        );

        TextColor playerColor = coloring.isEmpty()
            ? NamedTextColor.WHITE
            : coloring.getFirst().getNameColor(player);

        List<Prefixed> prefixes = Main.instance().getAppliances().stream()
            .filter(appliance -> appliance instanceof Prefixed)
            .map(appliance -> (Prefixed) appliance)
            .toList();

        ComponentBuilder<TextComponent, TextComponent.Builder> playerName = Component.text();
        prefixes.stream()
            .map(prefixed -> prefixed.getNamePrefix(player))
            .filter(Objects::nonNull)
            .forEach(prefix -> playerName
                .append(prefix)
                .append(ComponentUtil.clearedSpace())
        );

        if(Floodgate.isBedrock(player)) {
            playerName
                .append(
                    Component.text("\uD83C\uDFAE", NamedTextColor.GRAY)
                        .hoverEvent(HoverEvent.showText(Component.text(
                            String.format("%s spielt die Minecraft: Bedrock Edition", player.getName())
                        )))
                )
                .append(ComponentUtil.clearedSpace());
        }

        playerName.append(Component.text(player.getName(), playerColor));

        this.setGlobal(player, playerName.build());
    }

    private void setGlobal(Player player, Component component) {
        try {
            player.customName(component);
            player.displayName(component);
            player.playerListName(component);
        } catch(Exception e) {
            Main.instance().getLogger().log(Level.SEVERE, e, e::getMessage);
        }
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new DisplayNameUpdateListener());
    }
}
