package eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.infoBars;

import eu.mhsl.craftattack.spawn.core.Main;
import net.kyori.adventure.bossbar.BossBar;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.scheduler.BukkitTask;

import java.time.Duration;
import java.time.temporal.ChronoUnit;

public abstract class Bar {
    private BossBar bossBar;
    private final BukkitTask updateTask;

    public Bar() {
        long refreshRateInTicks = this.refresh().get(ChronoUnit.SECONDS) * Ticks.TICKS_PER_SECOND;
        this.updateTask = Bukkit.getScheduler().runTaskTimerAsynchronously(
            Main.instance(),
            this::update,
            refreshRateInTicks,
            refreshRateInTicks
        );
    }

    public BossBar getBossBar() {
        if(this.bossBar == null) this.bossBar = this.createBar();
        return this.bossBar;
    }

    private BossBar createBar() {
        return BossBar.bossBar(
            this.title(),
            this.correctedProgress(),
            this.color(),
            this.overlay()
        );
    }

    private void update() {
        if(this.bossBar == null) return;

        this.beforeRefresh();
        this.bossBar.name(this.title());
        this.bossBar.progress(this.correctedProgress());
        this.bossBar.color(this.color());
        this.bossBar.overlay(this.overlay());
    }

    public void stopUpdate() {
        this.updateTask.cancel();
    }

    private float correctedProgress() {
        return Math.clamp(this.progress(), 0, 1);
    }

    protected void beforeRefresh() {
    }

    protected abstract Duration refresh();
    protected abstract String name();

    protected abstract Component title();
    protected abstract float progress();
    protected abstract BossBar.Color color();
    protected abstract BossBar.Overlay overlay();
}
