package eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.infoBars;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.infoBars.bars.MsptBar;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.infoBars.bars.PlayerCounterBar;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.infoBars.bars.TpsBar;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;

public class InfoBars extends Appliance {
    private final NamespacedKey infoBarKey = new NamespacedKey(Main.instance(), "infobars");
    private final List<Bar> infoBars = List.of(
        new TpsBar(),
        new MsptBar(),
        new PlayerCounterBar()
    );

    public void showAll(Player player) {
        this.getStoredBars(player).forEach(bar -> this.show(player, bar));
    }

    public void hideAll(Player player) {
        this.getStoredBars(player).forEach(bar -> this.hide(player, bar));
        this.setStoredBars(player, List.of());
    }

    public void show(Player player, String bar) {
        this.validateBarName(bar);
        List<String> existingBars = new ArrayList<>(this.getStoredBars(player));
        existingBars.add(bar);
        player.showBossBar(this.getBarByName(bar).getBossBar());
        this.setStoredBars(player, existingBars);
    }

    public void hide(Player player, String bar) {
        this.validateBarName(bar);
        List<String> existingBars = new ArrayList<>(this.getStoredBars(player));
        existingBars.remove(bar);
        player.hideBossBar(this.getBarByName(bar).getBossBar());
        this.setStoredBars(player, existingBars);
    }

    private List<String> getStoredBars(Player player) {
        PersistentDataContainer container = player.getPersistentDataContainer();
        if(!container.has(this.infoBarKey)) return List.of();
        return container.get(this.infoBarKey, PersistentDataType.LIST.strings());
    }

    private void setStoredBars(Player player, List<String> bars) {
        player.getPersistentDataContainer().set(this.infoBarKey, PersistentDataType.LIST.strings(), bars);
    }

    private Bar getBarByName(String name) {
        return this.infoBars.stream()
            .filter(bar -> bar.name().equalsIgnoreCase(name))
            .findFirst()
            .orElse(null);
    }

    private void validateBarName(String name) {
        if(this.getBarByName(name) == null)
            throw new ApplianceCommand.Error(String.format("Ungültiger infobar name '%s'", name));
    }

    public List<Bar> getInfoBars() {
        return this.infoBars;
    }

    @Override
    public void onDisable() {
        this.infoBars.forEach(Bar::stopUpdate);
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new ShowPreviousBarsListener());
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new InfoBarCommand());
    }
}
