package eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.report;

import eu.mhsl.craftattack.spawn.common.api.repositories.ReportRepository;
import eu.mhsl.craftattack.spawn.common.api.repositories.VaroReportRepository;
import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.api.client.ReqResp;
import eu.mhsl.craftattack.spawn.common.api.repositories.CraftAttackReportRepository;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Optional;

public class Report extends Appliance {
    public static Component helpText() {
        return Component.text()
            .appendNewline()
            .append(Component.text(" Um einen Spieler zu melden, verwende ", NamedTextColor.GRAY)).appendNewline()
            .append(Component.text("/report", NamedTextColor.GOLD)).appendNewline()
            .append(Component.text("oder", NamedTextColor.GRAY)).appendNewline()
            .append(Component.text("/report <spieler> [grund]", NamedTextColor.GOLD)).appendNewline()
            .build();
    }

    public Report() {
        super("report");
    }

    public void reportToUnknown(@NotNull Player issuer) {
        CraftAttackReportRepository.ReportCreationInfo request = new CraftAttackReportRepository.ReportCreationInfo(issuer.getUniqueId(), null, "");
        Bukkit.getScheduler().runTaskAsynchronously(
            Main.instance(),
            () -> this.createReport(issuer, request)
        );
    }

    public void reportToKnown(@NotNull Player issuer, @NotNull String targetUsername, @Nullable String reason) {
        OfflinePlayer offlinePlayer = Bukkit.getOfflinePlayer(targetUsername);
        if(issuer.getUniqueId().equals(offlinePlayer.getUniqueId())) {
            issuer.sendMessage(Component.text("Du kannst dich nicht selbst reporten.", NamedTextColor.RED));
            return;
        }

        ReportRepository.ReportCreationInfo request = new ReportRepository.ReportCreationInfo(
            issuer.getUniqueId(),
            offlinePlayer.getUniqueId(),
            Optional.ofNullable(reason).orElse("")
        );
        Bukkit.getScheduler().runTaskAsynchronously(
            Main.instance(),
            () -> this.createReport(issuer, request)
        );
    }

    private void createReport(Player issuer, ReportRepository.ReportCreationInfo reportRequest) {
        ReqResp<ReportRepository.ReportUrl> createdReport = this.queryRepository(VaroReportRepository.class)
            .createReport(reportRequest);

        switch(createdReport.status()) {
            case 200: // varo-endpoint specific
            case 201:
                issuer.sendMessage(
                    Component.text()
                        .append(Component.text("\\/".repeat(20), NamedTextColor.DARK_GRAY))
                        .appendNewline()
                        .append(Component.text("⚠ Der Report muss über den folgenden Link fertiggestellt werden:", NamedTextColor.GOLD))
                        .appendNewline()
                        .appendNewline()
                        .append(
                            Component
                                .text(createdReport.data().url(), NamedTextColor.GRAY) // URL mit Weltkugel-Emoji
                                .clickEvent(ClickEvent.clickEvent(ClickEvent.Action.OPEN_URL, createdReport.data().url()))
                        )
                        .appendNewline()
                        .appendNewline()
                        .append(Component.text("Ohne das Fertigstellen des Reports wird dieser nicht bearbeitet!", NamedTextColor.DARK_RED))
                        .appendNewline()
                        .append(Component.text("/\\".repeat(20), NamedTextColor.DARK_GRAY))
                );
                break;

            case 400:
                issuer.sendMessage(
                    Component.text()
                        .append(Component.text("Der angegebene Nutzer ist in unserem System nicht bekannt.", NamedTextColor.RED))
                        .appendNewline()
                        .append(Component.text("Bist du sicher, dass du den Namen richtig geschrieben hast?", NamedTextColor.RED))
                        .appendNewline()
                        .append(Component.text("Du kannst dich alternativ jederzeit bei einem Admin melden.", NamedTextColor.GRAY))
                );
                break;

            case 401:
            default:
                Main.logger().warning("Failed to request Report: " + createdReport.status());
                issuer.sendMessage(
                    Component.text()
                        .append(Component.text("Interner Serverfehler beim anlegen des Reports.", NamedTextColor.RED))
                        .appendNewline()
                        .append(Component.text("Bitte melde dich bei einem Admin!", NamedTextColor.RED))
                );
                break;
        }
    }

    public void queryReports(Player issuer) {
        ReqResp<ReportRepository.PlayerReports> userReports = this.queryRepository(VaroReportRepository.class)
            .queryReports(issuer.getUniqueId());

        if(userReports.status() != 200) {
            Main.logger().warning("Failed to request Reports: " + userReports.status());
            issuer.sendMessage(
                Component.text()
                    .append(Component.text("Interner Serverfehler beim abfragen der Reports.", NamedTextColor.RED))
                    .appendNewline()
                    .append(Component.text("Bitte melde dich bei einem Admin!", NamedTextColor.RED))
            );
            return;
        }

        List<ReportRepository.PlayerReports.Report> reports = userReports
            .data()
            .from_self()
            .stream()
            .filter(report -> !report.draft())
            .toList()
            .reversed();

        if(reports.isEmpty()) {
            issuer.sendMessage(
                Component.text()
                    .append(Component.text("Du hast noch niemanden reportet.", NamedTextColor.RED))
                    .appendNewline()
                    .append(Component.text("Um jemanden zu melden, nutze /report", NamedTextColor.GRAY))
            );
            return;
        }

        ComponentBuilder<TextComponent, TextComponent.Builder> component = Component.text()
            .append(Component.newline())
            .append(Component.text("Von dir erstellte Reports: ", NamedTextColor.GOLD))
            .appendNewline();

        reports.forEach(report -> {
            component
                .append(Component.text(" - ", NamedTextColor.WHITE))
                .append(
                    report.reported() != null
                        ? Component.text(report.reported().username(), NamedTextColor.WHITE)
                        : Component.text("Unbekannt", NamedTextColor.YELLOW)
                )
                .append(Component.text(String.format(": %s", report.subject()), NamedTextColor.GRAY))
                .clickEvent(ClickEvent.openUrl(report.url()))
                .hoverEvent(HoverEvent.showText(Component.text("Klicke, um den Report einzusehen.", NamedTextColor.GOLD)));
            component.appendNewline();
        });

        issuer.sendMessage(component.build());
    }

    @Override
    @NotNull
    protected List<ApplianceCommand<?>> commands() {
        return List.of(
            new ReportCommand(),
            new ReportsCommand()
        );
    }
}
