package eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.datatypes;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.Settings;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.IntStream;
import java.util.stream.Stream;

public abstract class IntegerSetting extends Setting<Integer> {
    private final List<Integer> options;

    public IntegerSetting(Settings.Key key, int minimum, int maximum) {
        this(key, IntStream.range(minimum, maximum+1).boxed().toList());
    }

    public IntegerSetting(Settings.Key key, List<Integer> options) {
        super(key);
        this.options = options;
    }

    @Override
    public ItemMeta buildMeta(ItemMeta meta) {
        Component componentBefore = Component.text("  " + this.fillWithSpaces(this.options.getLast()));
        Component componentAfter = Component.text("  " + this.fillWithSpaces(this.options.getFirst()));
        int listIndex = this.options.indexOf(this.state);
        if(listIndex > 0) componentBefore = Component.text("  " + this.fillWithSpaces(this.options.get(listIndex-1)));
        if(listIndex < this.options.size()-1) componentAfter = Component.text("  " + this.fillWithSpaces(this.options.get(listIndex+1)));

        meta.displayName(Component.text(this.title(), NamedTextColor.WHITE));
        List<Component> lore = new ArrayList<>(Stream.of(
            Component.empty()
                .append(Component.text("Wert: ", NamedTextColor.DARK_GRAY)),
            Component.empty()
                .append(componentBefore.color(NamedTextColor.DARK_GRAY))
                .append(Component.text("  " + this.fillWithSpaces(this.state), NamedTextColor.GREEN))
                .append(componentAfter.color(NamedTextColor.DARK_GRAY)),
            Component.empty()
        ).toList());
        lore.addAll(this.buildDescription(this.description()));
        meta.lore(lore);
        return meta;
    }

    private String fillWithSpaces(Integer option) {
        String optionString = option.toString();
        int optionLength = optionString.length();
        int maxInteger = this.options.stream().mapToInt(value -> value).max().orElse(0);
        int maxLength = String.valueOf(maxInteger).length();
        int padding = maxLength - optionLength;

        int padEnd = padding / 2;
        int padStart = padding - padEnd;

        optionString = " ".repeat(padStart) + optionString + " ".repeat(padEnd);
        return optionString;
    }

    @Override
    protected void change(Player player, ClickType clickType) {
        int elementBefore = this.options.getLast();
        int elementAfter = this.options.getFirst();
        int listIndex = this.options.indexOf(this.state);
        if(listIndex > 0) elementBefore = this.options.get(listIndex-1);
        if(listIndex < this.options.size()-1) elementAfter = this.options.get(listIndex+1);

        if(clickType.equals(ClickType.LEFT)) this.state = elementBefore;
        if(clickType.equals(ClickType.RIGHT)) this.state = elementAfter;
    }

    @Override
    protected void fromStorage(PersistentDataContainer container) {
        this.state = container.has(this.getNamespacedKey())
            ? Integer.valueOf(Objects.requireNonNull(container.get(this.getNamespacedKey(), PersistentDataType.STRING)))
            : this.defaultValue();

        if(!this.options.contains(this.state)) this.state = this.defaultValue();
    }

    @Override
    protected void toStorage(PersistentDataContainer container, Integer value) {
        container.set(this.getNamespacedKey(), PersistentDataType.STRING, new Gson().toJson(value));
    }

    @Override
    public Class<?> dataType() {
        return Integer.class;
    }

    @Override
    public Integer state() {
        return this.state;
    }
}
