package eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.datatypes;

import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.Settings;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.RecordComponent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.IntStream;

public abstract class MultiBoolSetting<T> extends Setting<T> {
    private String cursorPosition;

    public MultiBoolSetting(Settings.Key key) {
        super(key);
    }

    @Retention(RetentionPolicy.RUNTIME)
    public @interface DisplayName {
        String value();
    }

    @Override
    public ItemMeta buildMeta(ItemMeta meta) {
        record SettingField(String name, String displayName, Boolean value) {
        }

        meta.displayName(Component.text(this.title(), NamedTextColor.WHITE));
        List<Component> lore = new ArrayList<>();
        lore.add(Component.text("Status: ", NamedTextColor.DARK_GRAY));

        lore.addAll(
            Arrays.stream(this.state.getClass().getRecordComponents())
                .map(component -> {
                    try {
                        Method method = this.state.getClass().getDeclaredMethod(component.getName());
                        Boolean value = (Boolean) method.invoke(this.state);

                        DisplayName annotation = component.getAnnotation(DisplayName.class);
                        String displayName = annotation != null
                            ? annotation.value()
                            : component.getName();

                        return new SettingField(component.getName(), displayName, value);
                    } catch(NoSuchMethodException | InvocationTargetException | IllegalAccessException e) {
                        throw new RuntimeException(e);
                    }
                })
                .map(field -> {
                    if(this.cursorPosition == null) this.cursorPosition = field.name;
                    boolean isSelected = field.name.equals(this.cursorPosition);
                    return Component.text()
                        .append(Component.text(
                            isSelected ? "> " : "  ",
                            isSelected ? NamedTextColor.GREEN : NamedTextColor.GRAY)
                        )
                        .append(Component.text(
                            field.displayName + ": ",
                            isSelected ? NamedTextColor.DARK_GREEN : NamedTextColor.GRAY)
                        )
                        .append(Component.text(
                            field.value ? "Aktiviert" : "Deaktiviert",
                            field.value ? NamedTextColor.GREEN : NamedTextColor.RED)
                        )
                        .build();
                })
                .toList()
        );
        lore.add(Component.empty());
        lore.addAll(this.buildDescription(this.description()));
        lore.add(Component.empty());
        lore.add(Component.text("Linksklick", NamedTextColor.AQUA).append(Component.text(" zum Wählen der Option", NamedTextColor.GRAY)));
        lore.add(Component.text("Rechtsklick", NamedTextColor.AQUA).append(Component.text(" zum Ändern des Wertes", NamedTextColor.GRAY)));
        meta.lore(lore);
        return meta;
    }

    @Override
    protected void change(Player player, ClickType clickType) {
        var recordComponents = this.state.getClass().getRecordComponents();

        int currentIndex = IntStream.range(0, recordComponents.length)
            .filter(i -> recordComponents[i].getName().equals(this.cursorPosition))
            .findFirst()
            .orElse(-1);

        if(clickType.equals(ClickType.LEFT)) {
            currentIndex = (currentIndex + 1) % recordComponents.length;
            this.cursorPosition = recordComponents[currentIndex].getName();
        } else if(clickType.equals(ClickType.RIGHT)) {
            try {
                Object[] values = Arrays.stream(recordComponents)
                    .map(rc -> {
                        try {
                            return this.state.getClass().getDeclaredMethod(rc.getName()).invoke(this.state);
                        } catch(NoSuchMethodException | IllegalAccessException | InvocationTargetException e) {
                            throw new RuntimeException(e);
                        }
                    })
                    .toArray();

                Method getter = this.state.getClass().getDeclaredMethod(this.cursorPosition);
                boolean currentValue = (Boolean) getter.invoke(this.state);
                values[currentIndex] = !currentValue;

                //noinspection unchecked
                this.state = (T) this.state.getClass().getConstructor(
                    Arrays.stream(recordComponents).map(RecordComponent::getType).toArray(Class[]::new)
                ).newInstance(values);
            } catch(NoSuchMethodException | InvocationTargetException | IllegalAccessException |
                    InstantiationException e) {
                throw new RuntimeException(e);
            }
        }
    }

    @Override
    protected void fromStorage(PersistentDataContainer container) {
        String data = container.has(this.getNamespacedKey())
            ? Objects.requireNonNull(container.get(this.getNamespacedKey(), PersistentDataType.STRING))
            : new Gson().toJson(this.defaultValue());

        try {
            //noinspection unchecked
            this.state = (T) new Gson().fromJson(data, this.dataType());
        } catch(Exception e) {
            this.state = this.defaultValue();
        }
    }

    @Override
    protected void toStorage(PersistentDataContainer container, T value) {
        container.set(this.getNamespacedKey(), PersistentDataType.STRING, new Gson().toJson(value));
    }

    @Override
    public T state() {
        return this.state;
    }
}
