package eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.datatypes;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.Settings;
import eu.mhsl.craftattack.spawn.core.util.text.ComponentUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Material;
import org.bukkit.NamespacedKey;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.bukkit.persistence.PersistentDataContainer;

import java.util.List;

public abstract class Setting<TDataType> {
    protected TDataType state;
    private final Settings.Key key;

    public Setting(Settings.Key key) {
        this.key = key;
    }

    public NamespacedKey getNamespacedKey() {
        return new NamespacedKey(Main.instance(), this.key.name());
    }

    public Settings.Key getKey() {
        return this.key;
    }

    public void initializeFromPlayer(Player p) {
        this.fromStorage(p.getPersistentDataContainer());
    }

    public void triggerChange(Player p, ClickType clickType) {
        this.change(p, clickType);
        this.toStorage(p.getPersistentDataContainer(), this.state());
    }

    public ItemStack buildItem() {
        ItemStack stack = new ItemStack(this.icon(), 1);
        stack.setItemMeta(this.buildMeta(stack.getItemMeta()));
        return stack;
    }

    protected List<TextComponent> buildDescription(String description) {
        return ComponentUtil.lineBreak(description, 50)
            .map(s -> Component.text(s, NamedTextColor.GRAY))
            .toList();
    }

    protected abstract String title();
    protected abstract String description();
    protected abstract Material icon();
    public abstract ItemMeta buildMeta(ItemMeta meta);
    protected abstract void change(Player player, ClickType clickType);
    protected abstract TDataType defaultValue();
    protected abstract void fromStorage(PersistentDataContainer container);
    protected abstract void toStorage(PersistentDataContainer container, TDataType value);
    public abstract Class<?> dataType();
    public abstract TDataType state();
}
