package eu.mhsl.craftattack.spawn.common.appliances.tooling.endPrevent;

import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.core.config.Configuration;
import org.bukkit.Bukkit;
import org.bukkit.World;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class EndPrevent extends Appliance {
    private final String endPreventKey = "endPrevent";
    private State endPreventState;
    private final World endWorld = Bukkit.getWorlds().stream().filter(world -> world.getEnvironment().equals(World.Environment.THE_END)).findFirst().orElseThrow();

    public enum State {
        OPEN,
        CLOSED,
        NO_OUTER
    }

    public EndPrevent() {
        super("endPrevent");
        this.endPreventState = State.valueOf(this.localConfig().getString(this.endPreventKey, State.OPEN.name()));
        this.updateEndBorder();
    }

    private void updateEndBorder() {
        if(this.endPreventState == State.NO_OUTER) this.endWorld.getWorldBorder().setSize(500);
        if(this.endPreventState == State.OPEN) this.endWorld.getWorldBorder().setSize(this.endWorld.getWorldBorder().getMaxSize());
    }

    public void setEndState(State state) {
        this.localConfig().set(this.endPreventKey, state.name());
        Configuration.saveChanges();
        this.endPreventState = state;
        this.updateEndBorder();
    }

    public boolean isEndClosed() {
        return this.endPreventState.equals(State.CLOSED);
    }

    public boolean isOnlyInner() {
        return this.endPreventState.equals(State.NO_OUTER);
    }

    public State getEndPreventState() {
        return this.endPreventState;
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new EndPreventListener());
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new EndPreventCommand());
    }
}
