package eu.mhsl.craftattack.spawn.common.appliances.tooling.restart;

import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.core.util.text.Countdown;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class Restart extends Appliance {
    private final Countdown countdown;

    public Restart() {
        this.countdown = new Countdown(60, this::format, this::announce, this::onDone);
    }

    public void scheduleRestart() {
        try {
            this.countdown.start();
        } catch(IllegalStateException e) {
            throw new ApplianceCommand.Error(e.getMessage());
        }
    }

    public void cancelRestart() {
        try {
            this.countdown.cancel();
            this.announce(Component.text("Der geplante Serverneustart wurde abgebrochen!", NamedTextColor.RED));
        } catch(IllegalStateException e) {
            throw new ApplianceCommand.Error(e.getMessage());
        }
    }

    private Component format(Countdown.AnnouncementData data) {
        return Component.text()
            .append(Component.text("Serverneustart in ", NamedTextColor.DARK_RED))
            .append(Component.text(data.count(), NamedTextColor.RED))
            .append(Component.text(" " + data.unit() + "!", NamedTextColor.DARK_RED))
            .build();
    }

    private void announce(Component message) {
        IteratorUtil.onlinePlayers(player -> player.sendMessage(message));
    }

    private void onDone() {
        Bukkit.shutdown();
    }

    @Override
    @NotNull
    protected List<ApplianceCommand<?>> commands() {
        return List.of(new ScheduleRestartCommand(), new CancelRestartCommand());
    }
}
