package eu.mhsl.craftattack.spawn.core.api.client;

import eu.mhsl.craftattack.spawn.core.Main;
import org.apache.commons.lang3.NotImplementedException;
import org.reflections.Reflections;

import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.reflect.InvocationTargetException;
import java.util.List;
import java.util.Set;

public class RepositoryLoader {
    private final List<Repository> repositories;

    /**
     * Defines a repository as an abstraction and will not be loaded by this RepositoryLoader
     */
    @Retention(RetentionPolicy.RUNTIME)
    public @interface Abstraction {
    }

    public RepositoryLoader() {
        Reflections reflections = new Reflections(Main.projectPackage);
        Set<Class<? extends Repository>> repositories = reflections.getSubTypesOf(Repository.class);

        this.repositories = repositories.stream()
            .filter(repository -> !repository.isAnnotationPresent(Abstraction.class))
            .map(repository -> {
                try {
                    return (Repository) repository.getDeclaredConstructor().newInstance();
                } catch(InstantiationException | IllegalAccessException | InvocationTargetException |
                        NoSuchMethodException e) {
                    throw new RuntimeException(e);
                }
            })
            .toList();
    }

    public <T> T getRepository(Class<T> clazz) {
        //noinspection unchecked
        return this.repositories.stream()
            .filter(clazz::isInstance)
            .map(repository -> (T) repository)
            .findFirst()
            .orElseThrow(() -> new NotImplementedException(String.format("Repository '%s' not found!", clazz.getSimpleName())));
    }

    public List<Repository> getRepositories() {
        return this.repositories;
    }
}
