package eu.mhsl.craftattack.spawn.core.appliance;

import eu.mhsl.craftattack.spawn.core.Main;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.command.Command;
import org.bukkit.command.CommandExecutor;
import org.bukkit.command.CommandSender;
import org.bukkit.command.TabCompleter;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Optional;

/**
 * Utility class which enables command name definition over a constructor.
 */
public abstract class ApplianceCommand<T extends Appliance> extends CachedApplianceSupplier<T> implements TabCompleter, CommandExecutor {
    public String commandName;
    protected Component errorMessage = Component.text("Fehler: ").color(NamedTextColor.RED);

    public ApplianceCommand(String command) {
        this.commandName = command;
    }

    public ApplianceCommand(String command, Component errorMessage) {
        this(command);
        this.errorMessage = errorMessage;
    }

    @Override
    public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        try {
            this.execute(sender, command, label, args);
        } catch(Error e) {
            sender.sendMessage(this.errorMessage.append(Component.text(e.getMessage())));
        } catch(Exception e) {
            sender.sendMessage(this.errorMessage.append(Component.text("Interner Fehler")));
            Main.logger().warning("Error executing appliance command " + this.commandName + ": " + e.getMessage());
            e.printStackTrace(System.err);
            return false;
        }
        return true;
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        return null;
    }

    protected List<String> tabCompleteReducer(List<String> response, String[] args) {
        return response.stream().filter(s -> s.startsWith(args[args.length - 1])).toList();
    }

    protected abstract void execute(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) throws Exception;

    /**
     * Utility class for command which can only be used as a Player. You can access the executing player with the getPlayer() method.
     */
    public static abstract class PlayerChecked<T extends Appliance> extends ApplianceCommand<T> {
        private Player player;
        private Component notPlayerMessage = Component.text("Dieser Command kann nur von Spielern ausgeführt werden!").color(NamedTextColor.RED);

        public PlayerChecked(String command) {
            super(command);
        }

        public PlayerChecked(String command, Component errorMessage) {
            super(command, errorMessage);
        }

        public PlayerChecked(String command, @Nullable Component errorMessage, Component notPlayerMessage) {
            super(command);
            this.errorMessage = Optional.ofNullable(errorMessage).orElse(this.errorMessage);
            this.notPlayerMessage = notPlayerMessage;
        }

        @Override
        public boolean onCommand(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
            if(!(sender instanceof Player)) {
                sender.sendMessage(this.notPlayerMessage);
                return false;
            }
            this.player = (Player) sender;
            return super.onCommand(sender, command, label, args);
        }

        public Player getPlayer() {
            return this.player;
        }
    }

    public static class Error extends RuntimeException {
        public Error(String message) {
            super(message);
        }
    }
}