package eu.mhsl.craftattack.spawn.core.util;

import org.bukkit.Bukkit;
import org.bukkit.GameRule;
import org.bukkit.World;
import org.bukkit.entity.Player;

import java.util.List;
import java.util.Map;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

public class IteratorUtil {
    public static void worlds(Consumer<World> world) {
        worlds(w -> w, world);
    }

    public static <T> void worlds(Function<World, T> selector, Consumer<T> selected) {
        Bukkit.getWorlds().forEach(world -> selected.accept(selector.apply(world)));
    }

    public static void onlinePlayers(Consumer<Player> player) {
        Bukkit.getOnlinePlayers().forEach(player);
    }

    public static void setGameRules(Map<GameRule<Boolean>, Boolean> rules, boolean inverse) {
        rules.forEach((gameRule, value) -> IteratorUtil.worlds(world -> world.setGameRule(gameRule, value ^ inverse)));
    }

    public static void times(int times, Runnable callback) {
        IntStream.range(0, times).forEach(value -> callback.run());
    }

    public static <T> void iterateListInGlobal(int sectionStart, List<T> list, BiConsumer<Integer, T> callback) {
        IntStream.range(sectionStart, sectionStart + list.size())
            .forEach(value -> callback.accept(value, list.get(value - sectionStart)));
    }

    public static void iterateLocalInGlobal(int sectionStart, int localLength, BiConsumer<Integer, Integer> callback) {
        IntStream.range(sectionStart, sectionStart + localLength)
            .forEach(value -> callback.accept(value, value + sectionStart));
    }

    public static <T> List<T> expandList(List<T> list, int targetSize, T defaultValue) {
        return Stream.concat(
            list.stream(),
            Stream.generate(() -> defaultValue).limit(Math.max(0, targetSize - list.size()))
        ).collect(Collectors.toList());
    }
}
