package eu.mhsl.craftattack.spawn.core.util.entity;

import eu.mhsl.craftattack.spawn.core.config.ConfigUtil;
import eu.mhsl.craftattack.spawn.core.config.Configuration;
import eu.mhsl.craftattack.spawn.core.util.world.ChunkUtils;
import org.bukkit.Location;
import org.bukkit.configuration.ConfigurationSection;
import org.bukkit.entity.EntityType;
import org.bukkit.entity.Villager;

import java.util.Objects;
import java.util.UUID;
import java.util.function.Consumer;

public class DisplayVillager {
    private final Location location;
    private Villager villager;

    public DisplayVillager(UUID uuid, Location location, Consumer<Villager> villagerCreator) {
        this.location = location;

        try {
            ChunkUtils.loadChunkAtLocation(this.location);
            this.villager = (Villager) this.location.getWorld().getEntity(uuid);
            Objects.requireNonNull(this.villager);
        } catch(NullPointerException | IllegalArgumentException e) {
            this.villager = this.getBaseVillager();
            villagerCreator.accept(this.villager);
        }

        this.villager.teleport(this.location);
    }

    public Villager getVillager() {
        return this.villager;
    }

    private Villager getBaseVillager() {
        Villager villager = (Villager) this.location.getWorld().spawnEntity(this.location, EntityType.VILLAGER);

        villager.setRemoveWhenFarAway(false);
        villager.setInvulnerable(true);
        villager.setPersistent(true);
        villager.setGravity(false);
        villager.setAI(false);
        villager.setCollidable(false);
        villager.setCustomNameVisible(true);
        return villager;
    }

    public static class ConfigBound {
        private final DisplayVillager villager;
        private final ConfigurationSection config;

        public ConfigBound(ConfigurationSection configurationSection, Consumer<Villager> villagerCreator) {
            this.config = configurationSection;

            Location location = ConfigUtil.Position.paseLocation(Objects.requireNonNull(this.config.getConfigurationSection("villagerLocation")));
            this.villager = new DisplayVillager(
                UUID.fromString(this.config.getString("uuid", UUID.randomUUID().toString())),
                location,
                villager -> {
                    this.config.set("uuid", villager.getUniqueId().toString());
                    Configuration.saveChanges();

                    villagerCreator.accept(villager);
                }
            );
        }

        public void updateLocation(Location location) {
            ConfigUtil.Position.writeLocation(
                Objects.requireNonNull(this.config.getConfigurationSection("villagerLocation")),
                location
            );
            Configuration.saveChanges();

            this.villager.getVillager().teleport(location);
        }

        public Villager getVillager() {
            return this.villager.getVillager();
        }

        public UUID getUniqueId() {
            return this.getVillager().getUniqueId();
        }
    }
}
