package eu.mhsl.craftattack.spawn.core.util.text;

import eu.mhsl.craftattack.spawn.core.util.statistics.NetworkMonitor;
import eu.mhsl.craftattack.spawn.core.util.statistics.ServerMonitor;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.format.TextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;

import java.awt.*;
import java.lang.management.OperatingSystemMXBean;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class ComponentUtil {
    public static TextComponent pleaseWait() {
        return Component.text("Bitte warte einen Augenblick...", NamedTextColor.GRAY);
    }

    public static Component clearedSpace() {
        return Component.text(" ").hoverEvent(Component.empty().asHoverEvent());
    }

    public static TextComponent appendWithNewline(Component a, Component b) {
        return Component.text().append(a.appendNewline().append(b)).build();
    }

    public static TextComponent appendWithSpace(Component a, Component b) {
        return Component.text().append(a).append(Component.text(" ")).append(b).build();
    }

    public static Stream<String> lineBreak(String text) {
        return lineBreak(text, 50);
    }

    public static Stream<String> lineBreak(String text, int charactersPerLine) {
        List<String> lines = new ArrayList<>();
        String[] words = text.split(" ");
        StringBuilder line = new StringBuilder();

        for(String word : words) {
            if(line.length() + word.length() + 1 > charactersPerLine) {
                lines.add(line.toString().trim());
                line = new StringBuilder();
            }
            line.append(word).append(" ");
        }

        if(!line.isEmpty()) {
            lines.add(line.toString().trim());
        }

        return lines.stream();
    }

    public static String lineBreakNL(String text, int charactersPerLine) {
        Stream<String> lines = lineBreak(text, charactersPerLine);
        return lines.collect(Collectors.joining("\n"));
    }

    public static Component getFormattedTickTimes(boolean detailed) {
        float mspt = ServerMonitor.getServerMSPT();
        float roundedMspt = Math.round(mspt * 100f) / 100f;
        int loadPercentage = (int) (Math.min(100, (mspt / 50.0) * 100));
        float roundedTPS = Math.round(Bukkit.getTPS()[0] * 100f) / 100f;

        TextColor percentageColor = ColorUtil.mapGreenToRed(loadPercentage, 80, 100, true);

        ComponentBuilder<TextComponent, TextComponent.Builder> tickTimes = Component.text()
            .append(Component.text("Serverlast: ", NamedTextColor.GRAY))
            .append(Component.text(loadPercentage + "% ", percentageColor))
            .appendNewline();

        if(detailed) {
            tickTimes
                .append(Component.text(roundedMspt + "mspt", ColorUtil.msptColor(mspt)))
                .append(Component.text(" | ", NamedTextColor.GRAY));
        }

        return tickTimes
            .append(Component.text(roundedTPS + "tps", ColorUtil.tpsColor(roundedTPS)))
            .build();
    }


    public static Component getFormattedPing(Player player) {
        int playerPing = player.getPing();
        int averagePing = Bukkit.getOnlinePlayers().stream()
            .map(Player::getPing).reduce(Integer::sum)
            .orElse(0) / Bukkit.getOnlinePlayers().size();

        return Component.text()
            .append(Component.text("Dein Ping: ", NamedTextColor.GRAY))
            .append(Component.text(playerPing + "ms", ColorUtil.mapGreenToRed(playerPing, 50, 200, true)))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text("Durschnitt: ", NamedTextColor.GRAY))
            .append(Component.text(averagePing + "ms", ColorUtil.mapGreenToRed(averagePing, 50, 200, true)))
            .build();
    }

    public static Component createRainbowText(String text, int step) {
        Component builder = Component.empty();
        int hue = 0;
        for(char c : text.toCharArray()) {
            TextColor color = TextColor.color(Color.getHSBColor((float) hue / 360, 1, 1).getRGB());
            builder = builder.append(Component.text(c).color(color));
            hue += step;
        }

        return builder;
    }

    public static Component getFormattedNetworkStats(NetworkMonitor.Traffic traffic, NetworkMonitor.Packets packets) {
        return Component.text()
            .append(Component.text(
                DataSizeConverter.convertBytesPerSecond(traffic.rxBytes()) + " ↓ " + NumberAbbreviation.abbreviateNumber(packets.rxCount()) + "pps",
                NamedTextColor.GREEN
            ))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text(
                DataSizeConverter.convertBytesPerSecond(traffic.txBytes()) + " ↑ " + NumberAbbreviation.abbreviateNumber(packets.rxCount()) + "pps",
                NamedTextColor.RED
            ))
            .build();
    }

    public static Component getFormattedSystemStats(OperatingSystemMXBean systemMonitor) {
        if(!(systemMonitor instanceof com.sun.management.OperatingSystemMXBean monitor))
            return Component.text("Could not get System information", NamedTextColor.DARK_GRAY);

        return Component.text()
            .append(Component.text("proc: ", NamedTextColor.GRAY))
            .append(Component.text(
                String.format("%.0f%%cpu", monitor.getProcessCpuLoad() * 100),
                NamedTextColor.GOLD
            ))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text(
                String.format("%s time", DataSizeConverter.formatCpuTimeToHumanReadable(monitor.getProcessCpuTime())),
                NamedTextColor.LIGHT_PURPLE
            ))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text(
                String.format(
                    "%s free, %s committed RAM",
                    DataSizeConverter.formatBytesToHumanReadable(monitor.getFreeMemorySize()),
                    DataSizeConverter.formatBytesToHumanReadable(monitor.getCommittedVirtualMemorySize())
                ),
                NamedTextColor.DARK_AQUA
            ))
            .appendNewline()
            .append(Component.text("sys: ", NamedTextColor.GRAY))
            .append(Component.text(
                String.format("%.0f%%cpu", monitor.getCpuLoad() * 100),
                NamedTextColor.GOLD
            ))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text(
                String.format(
                    "1min %.2f load avg (%.0f%%)",
                    monitor.getSystemLoadAverage(),
                    (monitor.getSystemLoadAverage() / monitor.getAvailableProcessors()) * 100
                ),
                NamedTextColor.LIGHT_PURPLE
            ))
            .append(Component.text(" | ", NamedTextColor.GRAY))
            .append(Component.text(
                String.format("%s total RAM", DataSizeConverter.formatBytesToHumanReadable(monitor.getTotalMemorySize())),
                NamedTextColor.DARK_AQUA
            ))
            .appendNewline()
            .append(Component.text(
                String.format(
                    "%s(%s) \uD83D\uDE80 on %s with %s cpu(s)",
                    monitor.getName(),
                    monitor.getVersion(),
                    monitor.getArch(),
                    monitor.getAvailableProcessors()
                ),
                NamedTextColor.GRAY
            ))
            .build();
    }
}
