package eu.mhsl.craftattack.spawn.craftattack.appliances.gameplay.customAdvancements;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import org.bukkit.Bukkit;
import org.bukkit.NamespacedKey;
import org.bukkit.advancement.Advancement;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class CustomAdvancements extends Appliance {
    record PendingAdvancement(UUID receiver, String advancement) {
    }

    private final List<PendingAdvancement> pendingAdvancements = new ArrayList<>();

    public void grantAdvancement(String advancementName, UUID playerUUID) {
        Player player = Bukkit.getPlayer(playerUUID);
        if(player == null) {
            this.addPendingAdvancement(playerUUID, advancementName);
            return;
        }

        try {
            NamespacedKey namespacedKey = Objects.requireNonNull(
                NamespacedKey.fromString("craftattack_advancements:craftattack/" + advancementName),
                String.format("NamespacedKey with '%s' is invalid!", advancementName)
            );
            Advancement advancement = Objects.requireNonNull(
                Bukkit.getAdvancement(namespacedKey),
                String.format("The advancement '%s' does not exist!", namespacedKey.asString())
            );
            player.getAdvancementProgress(advancement).awardCriteria("criteria");
        } catch(Exception e) {
            Main.logger().info("Advancement " + advancementName + " not found! (is Custom Advancements data pack loaded?)");
            throw e;
        }
    }

    public void applyPendingAdvancements(Player player) {
        if(this.pendingAdvancements.isEmpty()) return;
        List<PendingAdvancement> grantedAdvancements = this.pendingAdvancements.stream()
            .filter(pendingAdvancement -> pendingAdvancement.receiver.equals(player.getUniqueId())).toList();
        this.pendingAdvancements.removeAll(grantedAdvancements);
        grantedAdvancements.forEach(pendingAdvancement -> this.grantAdvancement(pendingAdvancement.advancement(), player.getUniqueId()));
    }

    private void addPendingAdvancement(UUID receiver, String advancement) {
        this.pendingAdvancements.add(new PendingAdvancement(receiver, advancement));
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(
            new CustomAdvancementsListener(),
            new ApplyPendingAdvancementsListener()
        );
    }
}
