package eu.mhsl.craftattack.spawn.craftattack.appliances.gameplay.knockDoor;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.Settings;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.datatypes.SelectSetting;
import org.bukkit.*;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.bukkit.metadata.FixedMetadataValue;
import org.bukkit.scheduler.BukkitRunnable;
import org.jetbrains.annotations.NotNull;

import java.util.List;

public class KnockDoor extends Appliance {
    @Override
    public void onEnable() {
        Settings.instance().declareSetting(KnockDoorSetting.class);
    }

    public void knockAtDoor(Player knockingPlayer, Block knockedBlock) {
        SelectSetting.Options.Option setting = Settings.instance().getSetting(knockingPlayer, Settings.Key.KnockDoors, SelectSetting.Options.Option.class);
        if(setting.is(KnockDoorSetting.disabled)) return;

        if(setting.is(KnockDoorSetting.knockSingleTime)) {
            this.playSound(knockedBlock);
        }

        if(setting.is(KnockDoorSetting.knockThreeTimes)) {
            String metadataKey = new NamespacedKey(Main.instance(), KnockDoor.class.getName()).getNamespace();
            if(knockingPlayer.hasMetadata(metadataKey)) return;
            knockingPlayer.setMetadata(metadataKey, new FixedMetadataValue(Main.instance(), 0));

            new BukkitRunnable() {
                @Override
                public void run() {
                    int timesKnocked = knockingPlayer.getMetadata(metadataKey).getFirst().asInt();
                    if(timesKnocked >= 3) {
                        knockingPlayer.removeMetadata(metadataKey, Main.instance());
                        this.cancel();
                        return;
                    }
                    KnockDoor.this.playSound(knockedBlock);
                    knockingPlayer.setMetadata(metadataKey, new FixedMetadataValue(Main.instance(), timesKnocked + 1));
                }
            }.runTaskTimer(Main.instance(), 0, 8);
        }
    }

    private void playSound(Block knockedBlock) {
        Location knockLocation = knockedBlock.getLocation();
        Sound sound = knockedBlock.getType() == Material.IRON_DOOR
            ? Sound.ENTITY_ZOMBIE_ATTACK_IRON_DOOR
            : Sound.ITEM_SHIELD_BLOCK;

        knockLocation.getWorld().playSound(knockLocation, sound, SoundCategory.PLAYERS, 1f, 1f);
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new KnockDoorListener());
    }
}
