package eu.mhsl.craftattack.spawn.craftattack.appliances.gameplay.outlawed;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.displayName.DisplayName;
import eu.mhsl.craftattack.spawn.craftattack.appliances.tooling.whitelist.Whitelist;
import eu.mhsl.craftattack.spawn.core.config.Configuration;
import eu.mhsl.craftattack.spawn.core.util.text.DisconnectInfo;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.*;

public class Outlawed extends Appliance implements DisplayName.Prefixed {
    public final int timeoutInMs = 1000 * 60 * 60 * 6;
    private final Map<UUID, Long> timeouts = new HashMap<>();

    public enum Status {
        DISABLED,
        VOLUNTARILY,
        FORCED
    }

    private final Map<Player, Status> playerStatusMap = new WeakHashMap<>();
    private final String voluntarilyEntry = "voluntarily";

    public Outlawed() {
        super("outlawed");
        Bukkit.getScheduler().runTaskTimerAsynchronously(
            Main.instance(),
            () -> this.playerStatusMap.forEach((player, status) -> {
                if(!player.isOnline()) return;
                if(status != Status.FORCED) return;
                try {
                    this.queryAppliance(Whitelist.class).fullIntegrityCheck(player);
                } catch(DisconnectInfo.Throwable e) {
                    e.getDisconnectScreen().applyKick(player);
                }
            }),
            20 * 60,
            20 * 60 * 5
        );
    }

    void switchLawStatus(Player player) throws OutlawChangeNotPermitted {
        if(this.getLawStatus(player).equals(Status.FORCED)) {
            throw new OutlawChangeNotPermitted("Dein Vogelfreistatus wurde als Strafe auferlegt und kann daher nicht verändert werden.");
        }

        if(this.isTimeout(player)) {
            throw new OutlawChangeNotPermitted("Du kannst deinen Vogelfreistatus nicht so schnell wechseln. Bitte warte einige Stunden bevor du umschaltest!");
        }

        this.setLawStatus(player, this.isOutlawed(player) ? Status.DISABLED : Status.VOLUNTARILY);
        this.setTimeout(player);
    }

    public void updateForcedStatus(Player player, boolean forced) {
        this.setLawStatus(player, forced ? Status.FORCED : this.getLawStatus(player) == Status.FORCED ? Status.DISABLED : this.getLawStatus(player));
    }

    public Status getLawStatus(Player player) {
        return this.playerStatusMap.computeIfAbsent(player, p -> {
            if(this.localConfig().getStringList(this.voluntarilyEntry).contains(p.getUniqueId().toString()))
                return Status.VOLUNTARILY;
            return Status.DISABLED;
        });
    }

    private void setLawStatus(Player player, Status status) {
        this.playerStatusMap.put(player, status);
        this.queryAppliance(DisplayName.class).update(player);

        List<String> newList = this.localConfig().getStringList(this.voluntarilyEntry);
        if(status.equals(Status.VOLUNTARILY)) {
            newList.add(player.getUniqueId().toString());
        } else {
            newList.remove(player.getUniqueId().toString());
        }
        this.localConfig().set(this.voluntarilyEntry, newList.stream().distinct().toList());
        Configuration.saveChanges();

    }

    public boolean isOutlawed(Player player) {
        return this.getLawStatus(player) != Status.DISABLED;
    }

    private boolean isTimeout(Player player) {
        return this.timeouts.getOrDefault(player.getUniqueId(), 0L) > System.currentTimeMillis() - this.timeoutInMs;
    }

    private void setTimeout(Player player) {
        this.timeouts.put(player.getUniqueId(), System.currentTimeMillis());
    }

    public Component getStatusDescription(Status status) {
        return switch(status) {
            case DISABLED -> Component.text("Vogelfreistatus inaktiv: ", NamedTextColor.GREEN)
                .append(Component.text("Es gelten die Standard Regeln!", NamedTextColor.GOLD));

            case VOLUNTARILY, FORCED -> Component.text("Vogelfreistatus aktiv: ", NamedTextColor.RED)
                .append(Component.text("Du darfst von jedem angegriffen und getötet werden!", NamedTextColor.GOLD));
        };
    }

    @Override
    public Component getNamePrefix(Player player) {
        if(this.isOutlawed(player)) {
            return Component.text("[☠]", NamedTextColor.RED)
                .hoverEvent(HoverEvent.showText(Component.text("Vogelfreie Spieler dürfen ohne Grund angegriffen werden!")));
        }

        return null;
    }

    @Override
    @NotNull
    protected List<ApplianceCommand<?>> commands() {
        return List.of(new OutlawedCommand());
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(new OutlawedReminderListener());
    }
}
