package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.afkTag;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.displayName.DisplayName;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashMap;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class AfkTag extends Appliance implements DisplayName.Prefixed {
    private final HashMap<UUID, Long> afkTimings = new HashMap<>();
    private static final int updateIntervalSeconds = 30;
    private static final int afkWhenMillis = 3 * 60 * 1000;

    @Override
    public void onEnable() {
        Bukkit.getScheduler().runTaskTimerAsynchronously(
            Main.instance(),
            this::checkAfkPlayers,
            Ticks.TICKS_PER_SECOND,
            Ticks.TICKS_PER_SECOND * updateIntervalSeconds
        );
    }

    public void resetTiming(Player player) {
        boolean wasAfk = this.isAfk(player);
        this.afkTimings.put(player.getUniqueId(), System.currentTimeMillis());
        if(wasAfk) this.updateAfkPrefix(player);
    }

    private void checkAfkPlayers() {
        this.afkTimings.keySet().stream()
            .map(Bukkit::getPlayer)
            .filter(Objects::nonNull)
            .filter(this::isAfk)
            .forEach(this::updateAfkPrefix);
    }

    private boolean isAfk(Player player) {
        if(player.isSleeping()) return false;

        long lastTimeActive = this.afkTimings.getOrDefault(player.getUniqueId(), 0L);
        long timeSinceLastActive = System.currentTimeMillis() - lastTimeActive;
        return timeSinceLastActive >= afkWhenMillis;
    }

    private void updateAfkPrefix(Player player) {
        Main.instance().getAppliance(DisplayName.class).update(player);
    }

    @Override
    public @Nullable Component getNamePrefix(Player player) {
        if(this.isAfk(player)) return Component.text("[ᵃᶠᵏ]", NamedTextColor.GRAY)
            .hoverEvent(HoverEvent.showText(Component.text("Der Spieler ist AFK")));

        return null;
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new AfkResetListener());
    }
}
