package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.event;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.api.client.ReqResp;
import eu.mhsl.craftattack.spawn.craftattack.api.repositories.EventRepository;
import eu.mhsl.craftattack.spawn.core.api.server.HttpServer;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.craftattack.appliances.gameplay.customAdvancements.Advancements;
import eu.mhsl.craftattack.spawn.craftattack.appliances.gameplay.customAdvancements.CustomAdvancements;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.core.api.HttpStatus;
import eu.mhsl.craftattack.spawn.core.util.entity.DisplayVillager;
import eu.mhsl.craftattack.spawn.core.util.listener.DismissInventoryOpenFromHolder;
import eu.mhsl.craftattack.spawn.core.util.listener.PlayerInteractAtEntityEventListener;
import eu.mhsl.craftattack.spawn.core.util.server.PluginMessage;
import eu.mhsl.craftattack.spawn.core.util.text.ComponentUtil;
import eu.mhsl.craftattack.spawn.core.util.text.Countdown;
import eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.event.command.*;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.entity.Villager;
import org.bukkit.event.Listener;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;
import org.jetbrains.annotations.NotNull;

import java.util.*;

public class Event extends Appliance {
    enum AdvertisementStatus {
        BEFORE,
        ADVERTISED,
        DONE
    }

    Countdown advertiseCountdown = new Countdown(
        120,
        announcementData -> Component.text()
            .append(ComponentUtil.createRainbowText("Event", 30))
            .append(Component.text(" Start in ", NamedTextColor.GOLD))
            .append(Component.text(announcementData.count(), NamedTextColor.AQUA))
            .append(Component.text(" " + announcementData.unit() + "!", NamedTextColor.GOLD))
            .build(),
        component -> IteratorUtil.onlinePlayers(player -> player.sendMessage(component)),
        () -> this.advertiseStatus = AdvertisementStatus.DONE
    );
    public DisplayVillager.ConfigBound villager;
    private boolean isOpen = false;
    private AdvertisementStatus advertiseStatus = AdvertisementStatus.BEFORE;
    private UUID roomId;
    private final List<Reward> pendingRewards = new ArrayList<>();

    record RewardConfiguration(String memorialMaterial, String memorialTitle, String memorialLore, List<UUID> memorials,
                               String material, Map<UUID, Integer> rewards) {
    }

    record Reward(UUID playerUuid, ItemStack itemStack) {
    }

    public Event() {
        super("event");
    }

    @Override
    public void onEnable() {
        this.villager = new DisplayVillager.ConfigBound(
            this.localConfig(),
            villager -> {
                villager.customName(Component.text("Events", NamedTextColor.GOLD));
                villager.setProfession(Villager.Profession.LIBRARIAN);
                villager.setVillagerType(Villager.Type.SNOW);
            }
        );
        this.isOpen = this.localConfig().getBoolean("enabled", false);
        if(this.isOpen) this.roomId = UUID.fromString(this.localConfig().getString("roomId", ""));
    }

    public void openEvent() {
        if(this.isOpen) throw new ApplianceCommand.Error("Es läuft derzeit bereits ein Event!");

        Bukkit.getScheduler().runTaskAsynchronously(Main.instance(), () -> {
            ReqResp<EventRepository.CreatedRoom> sessionResponse = this.queryRepository(EventRepository.class).createSession();

            if(sessionResponse.status() != HttpStatus.OK)
                throw new ApplianceCommand.Error("Event-Server meldet Fehler: " + sessionResponse.status());

            this.isOpen = true;
            this.roomId = sessionResponse.data().uuid();
        });
    }

    public void joinEvent(Player p) {
        if(!this.isOpen) {
            p.sendMessage(Component.text("Zurzeit ist kein Event geöffnet.", NamedTextColor.RED));
            return;
        }

        if(!p.hasPermission("admin") && this.advertiseStatus == AdvertisementStatus.BEFORE) {
            p.sendMessage(Component.text("Die Event befinden sich noch in der Vorbereitung.", NamedTextColor.RED));
            return;
        }

        if(!p.hasPermission("admin") && this.advertiseStatus == AdvertisementStatus.DONE) {
            p.sendMessage(Component.text("Die Events laufen bereits. Ein nachträgliches Beitreten ist nicht möglich.", NamedTextColor.RED));
            return;
        }

        Main.instance().getLogger().info("Verbinde mit eventserver: " + p.getName());
        p.sendMessage(Component.text("Authentifiziere...", NamedTextColor.GREEN));

        Bukkit.getScheduler().runTaskAsynchronously(Main.instance(), () -> {
            ReqResp<EventRepository.QueueRoom.Response> queueResponse = this.queryRepository(EventRepository.class)
                .queueRoom(new EventRepository.QueueRoom(p.getUniqueId(), this.roomId));

            if(queueResponse.status() != HttpStatus.OK || queueResponse.data().error() != null) {
                p.sendMessage(Component.text("Fehler beim Betreten: " + queueResponse.data().error(), NamedTextColor.RED));
                return;
            }

            p.sendMessage(Component.text("Betrete...", NamedTextColor.GREEN));
            PluginMessage.connect(p, this.localConfig().getString("connect-server-name"));
        });
    }

    public void endEvent() {
        if(!this.isOpen) throw new ApplianceCommand.Error("Es läuft derzeit kein Event!");
        this.isOpen = false;
    }

    private void rewardPlayers(RewardConfiguration rewardConfiguration) {
        rewardConfiguration.rewards.forEach((uuid, amount) -> {
            Reward reward = new Reward(uuid, new ItemStack(Objects.requireNonNull(Material.matchMaterial(rewardConfiguration.material)), amount));

            if(Bukkit.getPlayer(uuid) == null) {
                this.pendingRewards.add(reward);
                return;
            }
            this.giveReward(reward);
        });

        rewardConfiguration.memorials.forEach(uuid -> {
            ItemStack memorialItem = new ItemStack(Objects.requireNonNull(Material.matchMaterial(rewardConfiguration.memorialMaterial)));
            ItemMeta meta = memorialItem.getItemMeta();
            meta.displayName(Component.text(rewardConfiguration.memorialTitle, NamedTextColor.GOLD));
            meta.lore(List.of(Component.text(rewardConfiguration.memorialLore, NamedTextColor.AQUA)));
            memorialItem.setItemMeta(meta);
            Reward memorial = new Reward(uuid, memorialItem);

            Main.instance().getAppliance(CustomAdvancements.class).grantAdvancement(Advancements.participateEvent, uuid);

            if(Bukkit.getPlayer(uuid) == null) {
                this.pendingRewards.add(memorial);
                return;
            }
            this.giveReward(memorial);
        });

        rewardConfiguration.rewards.keySet().stream()
            .max(Comparator.comparing(rewardConfiguration.rewards::get))
            .ifPresent(uuid -> Main.instance().getAppliance(CustomAdvancements.class)
                .grantAdvancement(Advancements.winner, uuid));
    }

    private void giveReward(Reward reward) {
        Player player = Bukkit.getPlayer(reward.playerUuid);
        if(player == null) throw new RuntimeException("Cannot reward offline playerUuid!");

        Map<Integer, ItemStack> remaining = player.getInventory().addItem(reward.itemStack);
        Bukkit.getScheduler().runTask(
            Main.instance(),
            () -> remaining.values().forEach(remainingStack -> player.getWorld().dropItem(player.getLocation(), remainingStack))
        );
    }

    public void applyPendingRewards(Player player) {
        if(this.pendingRewards.isEmpty()) return;
        List<Reward> givenRewards = this.pendingRewards.stream().filter(reward -> reward.playerUuid.equals(player.getUniqueId())).toList();
        this.pendingRewards.removeAll(givenRewards);
        givenRewards.forEach(this::giveReward);
    }

    public void advertise() {
        this.advertiseCountdown.cancelIfRunning();
        this.advertiseStatus = AdvertisementStatus.ADVERTISED;
        IteratorUtil.onlinePlayers(player -> player.sendMessage(
            Component.text()
                .append(Component.text("-".repeat(10), NamedTextColor.GRAY)).appendNewline()
                .append(Component.text("Ein Event wurde gestartet!", NamedTextColor.GOLD)).appendNewline()
                .append(Component.text("Nutze "))
                .append(Component.text("/event", NamedTextColor.AQUA))
                .append(Component.text(", um dem Event beizutreten!")).appendNewline()
                .append(Component.text("-".repeat(10), NamedTextColor.GRAY)).appendNewline()
        ));
        this.advertiseCountdown.start();
    }

    @Override
    public void httpApi(HttpServer.ApiBuilder apiBuilder) {
        apiBuilder.post("reward", RewardConfiguration.class, (rewardConfiguration, request) -> {
            this.rewardPlayers(rewardConfiguration);
            return HttpServer.nothing;
        });
    }

    @Override
    @NotNull
    protected List<ApplianceCommand<?>> commands() {
        return List.of(
            new EventCommand(),
            new MoveEventVillagerCommand(),
            new EventOpenSessionCommand(),
            new EventEndSessionCommand(),
            new EventAdvertiseCommand()
        );
    }

    @Override
    @NotNull
    protected List<Listener> listeners() {
        return List.of(
            new ApplyPendingRewardsListener(),
            new PlayerInteractAtEntityEventListener(this.villager.getUniqueId(), playerInteractAtEntityEvent -> this.joinEvent(playerInteractAtEntityEvent.getPlayer())),
            new DismissInventoryOpenFromHolder(this.villager.getUniqueId())
        );
    }
}
