package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.feedback;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.api.client.ReqResp;
import eu.mhsl.craftattack.spawn.craftattack.api.repositories.FeedbackRepository;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.core.api.HttpStatus;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.ComponentBuilder;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Map;
import java.util.UUID;

public class Feedback extends Appliance {
    public Feedback() {
        super("feedback");
    }

    public void requestFeedback(String eventName, List<Player> receivers, @Nullable String question) {
        ReqResp<Map<UUID, String>> response = this.queryRepository(FeedbackRepository.class).createFeedbackUrls(
            new FeedbackRepository.Request(eventName, receivers.stream().map(Entity::getUniqueId).toList())
        );

        System.out.println(response.toString());
        System.out.println(response.status());

        if(response.status() != HttpStatus.CREATED) throw new RuntimeException();

        Component border = Component.text("-".repeat(40), NamedTextColor.GRAY);

        receivers.forEach(player -> {
            String feedbackUrl = response.data().get(player.getUniqueId());
            if(feedbackUrl == null) {
                Main.logger().warning(String.format("FeedbackUrl not found for player '%s' from backend!", player.getUniqueId()));
                return;
            }

            ComponentBuilder<TextComponent, TextComponent.Builder> message = Component.text()
                .append(border)
                .appendNewline();

            if(question != null) {
                message
                    .append(Component.text(question, NamedTextColor.GREEN))
                    .appendNewline()
                    .appendNewline();
            }

            message
                .append(Component.text("Klicke hier und gib uns Feedback, damit wir dein Spielerlebnis verbessern können!", NamedTextColor.DARK_GREEN)
                    .clickEvent(ClickEvent.openUrl(feedbackUrl)))
                .hoverEvent(HoverEvent.showText(Component.text("Klicke, um Feedback zu geben.")))
                .appendNewline()
                .append(border);

            player.sendMessage(message.build());
        });
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(
            new FeedbackCommand(),
            new RequestFeedbackCommand()
        );
    }
}
