package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.packSelect;

import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import eu.mhsl.craftattack.spawn.core.appliance.CachedApplianceSupplier;
import eu.mhsl.craftattack.spawn.core.util.inventory.HeadBuilder;
import eu.mhsl.craftattack.spawn.core.util.text.ComponentUtil;
import net.kyori.adventure.resource.ResourcePackInfo;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.text.serializer.plain.PlainTextComponentSerializer;
import org.bukkit.inventory.ItemStack;
import org.bukkit.inventory.meta.ItemMeta;

import java.util.*;
import java.util.stream.Collectors;

public class PackConfiguration extends CachedApplianceSupplier<PackSelect> {
    public record Pack(UUID id, String name, String description, String author, ResourcePackInfo info, String icon) {
        public ItemStack buildItem() {
            ItemStack stack = HeadBuilder.getCustomTextureHead(this.icon);
            ItemMeta meta = stack.getItemMeta();
            meta.itemName(Component.text(this.id.toString()));
            meta.displayName(Component.text(this.name(), NamedTextColor.GOLD));
            List<Component> lore = new ArrayList<>();
            lore.add(Component.text("Autor: ", NamedTextColor.DARK_GRAY).append(Component.text(this.author)));
            lore.add(Component.text(" "));
            lore.addAll(
                ComponentUtil.lineBreak(this.description())
                    .map(s -> Component.text(s, NamedTextColor.GRAY))
                    .toList()
            );
            lore.add(Component.text(" "));
            meta.lore(lore);
            stack.setItemMeta(meta);
            return stack;
        }

        public boolean equalsItem(ItemStack other) {
            String itemName = PlainTextComponentSerializer.plainText().serialize(other.getItemMeta().itemName());
            try {
                return UUID.fromString(itemName).equals(this.id);
            } catch(IllegalArgumentException ignored) {
                return false;
            }
        }
    }

    public record PackList(List<Pack> packs) {
        public Optional<Pack> findFromItem(ItemStack itemStack) {
            return this.packs.stream()
                .filter(pack -> pack.equalsItem(itemStack))
                .findFirst();
        }
    }

    public record SerializedPackList(List<UUID> packs) {
    }

    private final PackList packList;
    private final Gson gson = new GsonBuilder().create();

    private PackConfiguration() {
        this.packList = new PackList(new ArrayList<>());
    }

    private PackConfiguration(String data) {
        SerializedPackList serializedData = this.gson.fromJson(data, SerializedPackList.class);

        var availablePackMap = this.getAppliance().availablePacks.packs().stream()
            .collect(Collectors.toMap(PackConfiguration.Pack::id, pack -> pack));

        this.packList = new PackList(
            serializedData.packs().stream()
                .map(availablePackMap::get)
                .filter(Objects::nonNull)
                .collect(Collectors.toList())
        );

        if(this.packList.packs().isEmpty())
            throw new IllegalArgumentException("Serialized data does not contain any valid data!");
    }

    public static PackConfiguration deserialize(String data) {
        return new PackConfiguration(data);
    }

    public static PackConfiguration empty() {
        return new PackConfiguration();
    }

    public String serialize() {
        return this.gson.toJson(new SerializedPackList(this.packList.packs().stream().map(Pack::id).toList()));
    }

    public List<Pack> getPackList() {
        return this.packList.packs();
    }
}
