package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.packSelect;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.CachedApplianceSupplier;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import eu.mhsl.craftattack.spawn.core.util.inventory.ItemBuilder;
import eu.mhsl.craftattack.spawn.core.util.inventory.PlaceholderItems;
import eu.mhsl.craftattack.spawn.core.util.world.InteractSounds;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Bukkit;
import org.bukkit.Material;
import org.bukkit.entity.Player;
import org.bukkit.event.inventory.ClickType;
import org.bukkit.inventory.Inventory;
import org.bukkit.inventory.ItemStack;
import org.jetbrains.annotations.Nullable;

import java.util.Collections;
import java.util.List;
import java.util.stream.IntStream;

public class PackConfigurationInventory extends CachedApplianceSupplier<PackSelect> {
    private final PackConfiguration packConfiguration;
    private final Player inventoryOwner;
    private final Inventory inventory;

    private final ItemStack reset = ItemBuilder.of(Material.BARRIER)
        .displayName(Component.text("Zurücksetzen", NamedTextColor.RED))
        .lore("Alle gewählten Texturepacks werden entfernt")
        .build();

    private final ItemStack info = ItemBuilder.of(Material.OAK_HANGING_SIGN)
        .displayName(Component.text("Texturepacks", NamedTextColor.GOLD))
        .lore(
            "Wähle aus der oberen Liste eine oder mehrere Texturepacks aus. " +
                "Ändere die anzuwendende Reihenfolge durch Links/Rechtsklick in der unteren Liste. " +
                "Klicke auf Speichern um die Änderungen anzuwenden!"
        )
        .build();

    private final ItemStack save = ItemBuilder.of(Material.GREEN_WOOL)
        .displayName(Component.text("Anwenden", NamedTextColor.GREEN))
        .lore("Die Ausgewählten Texturepacks werden in der entsprechenden Reihenfolge angewandt. Das Anwenden kann durch den Download je nach Internetgeschwindigkeit eine Weile dauern.")
        .build();

    private final ItemStack unusedSlot = ItemBuilder.of(Material.LIME_STAINED_GLASS_PANE)
        .displayName(Component.text("Freier Slot", NamedTextColor.GREEN))
        .lore("Klicke auf ein Texturepack um es hinzuzufügen.")
        .build();


    public PackConfigurationInventory(PackConfiguration packConfiguration, Player inventoryOwner) {
        this.packConfiguration = packConfiguration;
        this.inventoryOwner = inventoryOwner;
        this.inventory = Bukkit.createInventory(null, 9 * 6, Component.text("Texturepacks"));
        this.draw();
    }

    public Inventory getInventory() {
        return this.inventory;
    }

    public void handleClick(@Nullable ItemStack clickedItem, int slot, ClickType clickType) {
        if(clickedItem == null) return;
        if(clickedItem.equals(this.reset)) this.reset();
        if(clickedItem.equals(this.save)) this.apply();

        if(slot >= 9 && slot < 9 * 4) {
            this.getAppliance().availablePacks.findFromItem(clickedItem)
                .ifPresent(this::toggle);
        }

        if(slot >= 9 * 5) {
            this.getAppliance().availablePacks.findFromItem(clickedItem)
                .ifPresent(pack -> {
                    switch(clickType) {
                        case RIGHT -> this.move(pack, true);
                        case LEFT -> this.move(pack, false);
                        case MIDDLE -> this.toggle(pack);
                    }
                });
        }
    }

    private void move(PackConfiguration.Pack pack, boolean moveToRight) {
        List<PackConfiguration.Pack> packs = this.packConfiguration.getPackList();
        int index = packs.indexOf(pack);

        if(index != -1) {
            int newIndex = moveToRight ? index + 1 : index - 1;
            if(newIndex >= 0 && newIndex < packs.size()) Collections.swap(packs, index, newIndex);
        }

        InteractSounds.of(this.inventoryOwner).click();
        this.draw();
    }

    private void toggle(PackConfiguration.Pack pack) {
        if(this.packConfiguration.getPackList().contains(pack)) {
            this.packConfiguration.getPackList().remove(pack);
        } else {
            this.packConfiguration.getPackList().add(pack);
        }

        InteractSounds.of(this.inventoryOwner).click();
        this.draw();
    }

    private void reset() {
        this.packConfiguration.getPackList().clear();
        InteractSounds.of(this.inventoryOwner).delete();
        this.draw();
    }

    private void apply() {
        this.inventoryOwner.closeInventory();
        InteractSounds.of(this.inventoryOwner).success();
        Bukkit.getScheduler().runTask(Main.instance(), () -> this.getAppliance().setPack(this.inventoryOwner, this.packConfiguration));
    }

    private void draw() {
        this.inventory.clear();

        this.inventory.setItem(0, this.packConfiguration.getPackList().isEmpty() ? PlaceholderItems.grayStainedGlassPane : this.reset);
        IteratorUtil.times(3, () -> this.inventory.addItem(PlaceholderItems.grayStainedGlassPane));
        this.inventory.setItem(4, this.info);
        IteratorUtil.times(3, () -> this.inventory.addItem(PlaceholderItems.grayStainedGlassPane));
        this.inventory.setItem(8, this.save);

        IteratorUtil.iterateListInGlobal(
            9,
            this.getAppliance().availablePacks.packs().stream()
                .filter(pack -> !this.packConfiguration.getPackList().contains(pack))
                .limit(9 * 3)
                .map(pack -> {
                    ItemBuilder stack = ItemBuilder.of(pack.buildItem());
                    if(this.packConfiguration.getPackList().contains(pack)) stack.glint();
                    return stack.build();
                })
                .toList(),
            this.inventory::setItem
        );

        IntStream.range(9 * 4, 9 * 5)
            .forEach(slot -> this.inventory.setItem(slot, PlaceholderItems.grayStainedGlassPane));

        IteratorUtil.iterateListInGlobal(
            9 * 5,
            IteratorUtil.expandList(
                IntStream.range(0, Math.min(this.packConfiguration.getPackList().size(), 9))
                    .mapToObj(i -> {
                        PackConfiguration.Pack pack = this.packConfiguration.getPackList().get(i);
                        ItemBuilder builder = ItemBuilder.of(pack.buildItem());

                        builder
                            .displayName(existing -> Component.text(String.format("#%s ", i + 1), NamedTextColor.LIGHT_PURPLE).append(existing))
                            .appendLore(Component.text("➡ Rechtsklick um nach Rechts zu verschieben", NamedTextColor.AQUA))
                            .appendLore(Component.text("⬅ Linksklick um nach Links zu verschieben", NamedTextColor.AQUA))
                            .appendLore(Component.text("\uD83D\uDDD1 Mausradklick um zu entfernen", NamedTextColor.AQUA));
                        return builder.build();
                    })
                    .toList(),
                9,
                this.unusedSlot
            ),
            this.inventory::setItem
        );

    }
}
