package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.packSelect;

import eu.mhsl.craftattack.spawn.core.Main;
import net.kyori.adventure.resource.ResourcePackInfo;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.InputStream;
import java.net.HttpURLConnection;
import java.net.URI;
import java.security.MessageDigest;
import java.util.UUID;
import java.util.concurrent.CompletableFuture;

class ResourcePackInfoFactory {

    private static boolean isValidHash(@Nullable String hash) {
        return hash != null && hash.length() == 40;
    }

    public static @NotNull CompletableFuture<ResourcePackInfo> createResourcePackInfo(@NotNull URI resourcePackUrl, @Nullable String hash) {
        if(isValidHash(hash)) {
            return CompletableFuture.completedFuture(
                ResourcePackInfo.resourcePackInfo(UUID.nameUUIDFromBytes(hash.getBytes()), resourcePackUrl, hash)
            );
        }

        return CompletableFuture.supplyAsync(() -> {
            try {
                Main.logger().info(String.format("Start calculating SHA1 Hash of %s", resourcePackUrl));
                HttpURLConnection connection = (HttpURLConnection) resourcePackUrl.toURL().openConnection();
                connection.setRequestMethod("GET");
                connection.connect();

                try(InputStream inputStream = connection.getInputStream()) {
                    MessageDigest sha1Digest = MessageDigest.getInstance("SHA-1");
                    byte[] buffer = new byte[1024];
                    int bytesRead;
                    while((bytesRead = inputStream.read(buffer)) != -1) {
                        sha1Digest.update(buffer, 0, bytesRead);
                    }
                    String sha1Hex = bytesToHex(sha1Digest.digest());

                    Main.logger().info(String.format("Calculating SHA1 Hash of %s completed: %s", resourcePackUrl, sha1Hex));
                    return ResourcePackInfo.resourcePackInfo(UUID.nameUUIDFromBytes(sha1Hex.getBytes()), resourcePackUrl, sha1Hex);
                }
            } catch(Exception e) {
                String error = String.format("Error whilst SHA1 calculation of %s: %s", resourcePackUrl, e.getMessage());
                Main.logger().warning(error);
                throw new RuntimeException(error);
            }
        });
    }

    private static String bytesToHex(byte[] bytes) {
        StringBuilder hexString = new StringBuilder(bytes.length * 2);
        for(byte b : bytes) {
            hexString.append(String.format("%02x", b));
        }
        return hexString.toString();
    }
}
