package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.sleepTag;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.displayName.DisplayName;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.HashSet;
import java.util.List;
import java.util.Set;

public class SleepTag extends Appliance implements DisplayName.Prefixed {
    private final Set<Player> sleepingPlayers = new HashSet<>();

    @Override
    public void onEnable() {
        Bukkit.getScheduler().runTaskTimerAsynchronously(
            Main.instance(),
            this::cleanup,
            Ticks.TICKS_PER_SECOND * 60,
            Ticks.TICKS_PER_SECOND * 60
        );
    }

    public void updateSleeping(Player player, boolean isSleeping) {
        if(isSleeping) {
            this.sleepingPlayers.add(player);
        } else {
            this.sleepingPlayers.remove(player);
        }
        this.updateDisplayName(player);
    }

    private void updateDisplayName(Player player) {
        Main.instance().getAppliance(DisplayName.class).update(player);
    }

    private void cleanup() {
        List<Player> invalidEntries = this.sleepingPlayers.stream()
            .filter(player -> !player.isConnected())
            .filter(player -> !player.isSleeping())
            .toList();

        invalidEntries.forEach(this.sleepingPlayers::remove);
        invalidEntries.forEach(this::updateDisplayName);
    }

    @Override
    public @Nullable Component getNamePrefix(Player player) {
        if(this.sleepingPlayers.contains(player))
            return Component.text("[\uD83D\uDCA4]", NamedTextColor.GRAY)
                .hoverEvent(HoverEvent.showText(Component.text("Der Spieler liegt in einem Bett")));

        return null;
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new SleepStateChangeListener());
    }
}
