package eu.mhsl.craftattack.spawn.craftattack.appliances.metaGameplay.yearRank;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.displayName.DisplayName;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.TextComponent;
import net.kyori.adventure.text.event.ClickEvent;
import net.kyori.adventure.text.event.HoverEvent;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.io.File;
import java.util.*;

public class YearRank extends Appliance implements DisplayName.Prefixed {
    record CraftAttackYear(String name) {
    }

    private final Map<UUID, List<CraftAttackYear>> rankMap = new HashMap<>();

    @Override
    public void onEnable() {
        File folder = new File(Main.instance().getDataFolder(), "yearRank");

        //noinspection ResultOfMethodCallIgnored
        folder.mkdirs();

        Optional<File[]> dataFolders = Optional.ofNullable(folder.listFiles());
        if(dataFolders.isEmpty()) return;

        List.of(dataFolders.get()).forEach(playerDataFolder -> {
            Optional<File[]> datFiles = Optional.ofNullable(playerDataFolder.listFiles());
            if(datFiles.isEmpty()) return;

            CraftAttackYear craftAttackYear = new CraftAttackYear(playerDataFolder.getName());

            Arrays.stream(datFiles.get())
                .map(file -> file.getName().split("\\.")[0])
                .distinct()
                .map(UUID::fromString)
                .peek(uuid -> this.rankMap.computeIfAbsent(uuid, p -> new ArrayList<>()))
                .forEach(uuid -> this.rankMap.get(uuid).add(craftAttackYear));
        });
    }

    @Override
    public @Nullable Component getNamePrefix(Player player) {
        if(!this.rankMap.containsKey(player.getUniqueId())) return null;
        int yearCount = this.rankMap.get(player.getUniqueId()).size();
        if(yearCount <= 3) return null;

        return Component.text()
            .append(Component.text("[\uD83C\uDF1F]", NamedTextColor.GOLD))
            .hoverEvent(HoverEvent.showText(
                Component.text(String.format("Langzeitspieler: %s ist bereits seit %d Jahren dabei!", player.getName(), yearCount))
            ))
            .build();
    }

    public Component listYearRanks() {
        TextComponent.Builder builder = Component.text();
        builder.append(Component.text("Top 30 Spieler: ", NamedTextColor.GOLD));
        this.rankMap.keySet().stream()
            .map(uuid -> Map.entry(uuid, this.rankMap.get(uuid).size()))
            .sorted(Map.Entry.comparingByValue(Comparator.reverseOrder()))
            .limit(30)
            .forEach(entry -> builder
                .appendNewline()
                .append(Component.text(entry.getKey().toString(), NamedTextColor.GRAY)
                    .hoverEvent(HoverEvent.showText(Component.text(entry.getKey().toString())))
                    .clickEvent(ClickEvent.copyToClipboard(entry.getKey().toString())))
                .append(Component.text(": "))
                .append(Component.text(entry.getValue(), NamedTextColor.GOLD)));

        builder
            .appendNewline()
            .appendNewline()
            .append(Component.text("Übersischt:", NamedTextColor.GOLD));

        this.rankMap.values().stream()
            .mapMulti(Iterable::forEach)
            .filter(o -> o instanceof CraftAttackYear)
            .map(o -> (CraftAttackYear) o)
            .distinct()
            .forEach(craftAttackYear -> builder
                .appendNewline()
                .append(Component.text(craftAttackYear.name, NamedTextColor.GRAY))
                .append(Component.text(": "))
                .append(Component.text(this.rankMap.keySet().stream()
                    .filter(uuid -> this.rankMap.get(uuid).contains(craftAttackYear)).count())));

        return builder.build();
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new YearRankCommand());
    }
}
