package eu.mhsl.craftattack.spawn.varo.appliances.internal.teamTasks;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.VaroTeam;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class TeamTasks extends Appliance {
    public enum Type {
        /**
         * Task for kicking Team after the desired Playtime
         */
        TIME_KICK
    }

    private final Map<VaroTeam, Map<Type, Task>> tasks = new HashMap<>();

    private Map<Type, Task> getTeamTasks(VaroTeam team) {
        return this.tasks.computeIfAbsent(team, varoTeam -> new HashMap<>());
    }

    public Map<Type, Task> getRunningTeamTasks(VaroTeam team) {
        return this.getTeamTasks(team).entrySet().stream()
            .filter(entry -> entry.getValue().isTaskRunning())
            .collect(Collectors.toMap(Map.Entry::getKey, Map.Entry::getValue));
    }

    public void cancelTeamTasks(VaroTeam team) {
        Main.logger().info(String.format("All TeamTasks for Team %s were cancelled: %s", team.name, this.getRunningTeamTasks(team)));
        this.getTeamTasks(team).forEach((type, task) -> task.stopTask());
    }

    public void addTask(VaroTeam team, Type type, Task runnable) {
        if(this.getTeamTasks(team).containsKey(type) && this.getTeamTasks(team).get(type).isTaskRunning()) {
            throw new IllegalStateException(String.format("Task %s for Team %s was already running!", type.name(), team.name));
        }

        this.getTeamTasks(team).put(type, runnable);
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new TeamTasksCommand());
    }
}
