package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.fightDetector;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.Teams;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.VaroTeam;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

public class FightDetector extends Appliance {
    public static final long FIGHT_TIMEOUT = 60 * 1000;
    private static final long BLOCK_RADIUS = 30;

    public final Map<VaroTeam, Long> fights = new HashMap<>();

    public FightDetector() {
        Bukkit.getScheduler().runTaskTimer(
            Main.instance(),
            () -> {
                var teamFights = this.fights.keySet().stream()
                    .filter(this::isInFight)
                    .toList();
                if(teamFights.isEmpty()) return;
                Main.logger().info(String.format(
                    "There are %d Teams in Fight: %s",
                    teamFights.size(),
                    teamFights.stream()
                        .map(varoTeam -> String.format(
                            "%s[%s]",
                            varoTeam.name,
                            varoTeam.members.stream()
                                .map(member -> member.player.getName())
                                .collect(Collectors.joining(","))))
                        .collect(Collectors.joining(", "))
                ));
            },
            Ticks.TICKS_PER_SECOND * 15,
            Ticks.TICKS_PER_SECOND * 15
        );

        Bukkit.getScheduler().runTaskTimer(
            Main.instance(),
            this::detectNearbyFights,
            Ticks.TICKS_PER_SECOND,
            Ticks.TICKS_PER_SECOND
        );
    }

    private void detectNearbyFights() {
        var players = Bukkit.getOnlinePlayers();
        Bukkit.getScheduler().runTaskAsynchronously(Main.instance(), () -> {
            for (Player player : players) {
                VaroTeam ownTeam = this.queryAppliance(Teams.class).getTeamFromPlayer(player.getUniqueId());
                if (ownTeam == null) continue;

                for (Player otherPlayer : players) {
                    if (player.equals(otherPlayer)) continue;

                    VaroTeam otherTeam = this.queryAppliance(Teams.class).getTeamFromPlayer(otherPlayer.getUniqueId());
                    if (otherTeam == null || ownTeam.equals(otherTeam)) continue;

                    if (player.getLocation().distance(otherPlayer.getLocation()) <= BLOCK_RADIUS) {
                        this.setInFight(ownTeam);
                        this.setInFight(otherTeam);
                    }
                }
            }
        });
    }

    public boolean isInFight(VaroTeam team) {
        Long lastFightTime = this.fights.get(team);
        if(lastFightTime == null) return false;
        return (System.currentTimeMillis() - lastFightTime <= FIGHT_TIMEOUT);
    }

    public void setInFight(VaroTeam team) {
        this.fights.put(team, System.currentTimeMillis());
    }
    public void setInFight(Player player) {
        this.setInFight(this.queryAppliance(Teams.class).getTeamFromPlayer(player.getUniqueId()));
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(
            new FightOnDistanceListener(),
            new FightOnInteractionListener()
        );
    }
}
