package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.joinProtection;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.util.IteratorUtil;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Cancellable;
import org.bukkit.event.Listener;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public class JoinProtection extends Appliance {
    public static final int resistanceDuration = 10;

    static final class Options {
        public boolean wasOnGround = false;
        public Long joinTime = System.currentTimeMillis();
    }
    private final Map<UUID, Options> protectedPlayers = new HashMap<>();

    public void addProtection(Player player) {
        if(player.isOp()) return;

        this.protectedPlayers.put(player.getUniqueId(), new Options());
        PotionEffect resistance = new PotionEffect(PotionEffectType.RESISTANCE, Ticks.TICKS_PER_SECOND * resistanceDuration, 1);
        PotionEffect blindness = new PotionEffect(PotionEffectType.DARKNESS, Ticks.TICKS_PER_SECOND * 3, 1);
        player.addPotionEffects(List.of(resistance, blindness));

        Bukkit.getScheduler().runTaskTimer(
            Main.instance(),
            this::updateStatus,
            Ticks.TICKS_PER_SECOND,
            Ticks.TICKS_PER_SECOND
        );
    }

    @Nullable JoinProtection.Options getOptions(Player player) {
        return this.protectedPlayers.get(player.getUniqueId());
    }

    public boolean isNotProtected(Player player) {
        Options options = this.protectedPlayers.get(player.getUniqueId());
        if(options == null) return true;
        return options.joinTime <= System.currentTimeMillis() - (resistanceDuration * 1000L);
    }

    public void cancelEvent(Player player, Cancellable event) {
        if(this.isNotProtected(player)) return;
        event.setCancelled(true);
    }

    public void updateStatus() {
        IteratorUtil.onlinePlayers(player -> {
            Options options = this.protectedPlayers.get(player.getUniqueId());
            if(options == null) return;

            if(this.isNotProtected(player)) {
                this.protectedPlayers.remove(player.getUniqueId());
            }

            int secondsLeft = Math.abs((int) ((System.currentTimeMillis() - options.joinTime) / 1000) - resistanceDuration);
            player.sendActionBar(
                Component.text(
                    secondsLeft > 0
                        ? String.format("Du bist in %d Sekunden angreifbar!", secondsLeft)
                        : "Du jetzt angreifbar!",
                    NamedTextColor.RED
                )
            );
        });
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new JoinProtectionListener());
    }
}
