package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.playTimer;

import com.google.common.reflect.TypeToken;
import com.google.gson.Gson;
import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.api.server.HttpServer;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.Teams;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.VaroTeam;
import org.jetbrains.annotations.NotNull;

import java.io.IOException;
import java.io.Reader;
import java.io.Writer;
import java.lang.reflect.Type;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.NoSuchElementException;

public class PlayTimer extends Appliance {
    public static final int PLAYTIME_MINUTES = 30;

    private final Map<String, Integer> joinTickets = new HashMap<>();
    private final Path saveFile = Paths.get(Main.instance().getDataFolder().getAbsolutePath() + "/playtime.json");

    public PlayTimer() {
        this.load();
    }

    private void load() {
        if (!Files.exists(this.saveFile)) return;
        try (Reader reader = Files.newBufferedReader(this.saveFile)) {
            Type type = new TypeToken<Map<String, Object>>() {}.getType();
            Map<String, Object> data = new Gson().fromJson(reader, type);
            @SuppressWarnings("unchecked") Map<String, Double> ticketMap = (Map<String, Double>) data.get("tickets");

            if (ticketMap != null) {
                for (Map.Entry<String, Double> entry : ticketMap.entrySet()) {
                    this.joinTickets.put(entry.getKey(), entry.getValue().intValue());
                }
            }
        } catch (IOException e) {
            Main.logger().warning("Failed reading playtime from teams: " + e.getMessage());
        }
    }

    private void save() {
        try {
            Files.createDirectories(this.saveFile.getParent());
            try (Writer writer = Files.newBufferedWriter(this.saveFile)) {
                new Gson().toJson(Map.of("tickets", this.joinTickets), writer);
            }
        } catch (IOException e) {
            Main.logger().warning("Failed to save playtime for teams: " + e.getMessage());
        }
    }

    public void incrementAll() {
        this.joinTickets.replaceAll((n, v) -> this.joinTickets.get(n) + 1);
        this.save();
    }
    public void setTickets(VaroTeam team, int amount) {
        this.joinTickets.put(team.name, amount);
        this.save();
    }

    public int getTickets(VaroTeam team) {
        return this.joinTickets.getOrDefault(team.name, 1);
    }

    public boolean tryConsumeTicket(VaroTeam team) {
        String teamName = team.name;
        int current = this.joinTickets.getOrDefault(teamName, 1);

        if (current <= 0) return false;

        this.joinTickets.put(teamName, current - 1);
        this.save();
        return true;
    }


    @Override
    public void httpApi(HttpServer.ApiBuilder apiBuilder) {
        record Ticket(String team, int tickets)  {}
        apiBuilder.get("tickets", request -> {
            String teamName = request.queryParamsSafe("team");
            VaroTeam team = Main.instance().getAppliance(Teams.class).findTeamByName(teamName);
            if(team == null) throw new NoSuchElementException("Team not found!");

            return new Ticket(team.name, this.getTickets(team));
        });
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new PlayTimerCommand());
    }
}
