package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.playTimer;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.Teams;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams.VaroTeam;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.stream.Stream;

public class PlayTimerCommand extends ApplianceCommand<PlayTimer> {
    public PlayTimerCommand() {
        super("playTimer");
    }

    @Override
    protected void execute(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) throws Exception {
        if (args.length < 3) throw new Error("Usage: playTimer <user|team> <identifier> <get|set> [amount]");

        String mode = args[0].toLowerCase();
        String identifier = args[1];
        String action = args[2].toLowerCase();

        Teams teamAppliance = Main.instance().getAppliance(Teams.class);
        VaroTeam team = switch (mode) {
            case "user" -> {
                OfflinePlayer player = Bukkit.getOfflinePlayer(identifier);
                yield teamAppliance.getTeamFromPlayer(player.getUniqueId());
            }
            case "team" -> teamAppliance.findTeamByName(identifier);
            case "incallbyone" -> {
                this.getAppliance().incrementAll();
                throw new Error("Incremented all Teams by one!");
            }
            default -> throw new Error("Ungültiger Modus: " + mode + ". Erlaubt: user | team");
        };

        if(team == null) throw new Error("Team nicht gefunden.");

        switch (action) {
            case "get" -> {
                int ticketCount = this.getAppliance().getTickets(team);
                sender.sendMessage(String.format("Team %s hat %d tickets!", team.name, ticketCount));
            }
            case "set" -> {
                if (args.length < 4) throw new Error("Usage: playTimer <user|team> <identifier> set <amount>");
                int amount = Integer.parseInt(args[3]);
                this.getAppliance().setTickets(team, amount);
                sender.sendMessage("Tickets wurden gesetzt!");
            }
            default -> throw new Error("Ungültige Aktion: " + action + ". Erlaubt: get | set");
        }
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        List<VaroTeam> teams = Main.instance().getAppliance(Teams.class).getAllTeams();

        return switch (args.length) {
            case 1 -> Stream.of("user", "team", "incAllByOne")
                .filter(opt -> opt.startsWith(args[0].toLowerCase()))
                .toList();

            case 2 -> {
                if (args[0].equalsIgnoreCase("user")) {
                    yield Bukkit.getOnlinePlayers().stream()
                        .map(Player::getName)
                        .filter(name -> name.toLowerCase().startsWith(args[1].toLowerCase()))
                        .toList();
                } else if (args[0].equalsIgnoreCase("team")) {
                    yield teams.stream()
                        .map(team -> team.name)
                        .filter(name -> name.toLowerCase().startsWith(args[1].toLowerCase()))
                        .toList();
                } else {
                    yield List.of();
                }
            }

            case 3 -> Stream.of("get", "set")
                .filter(opt -> opt.startsWith(args[2].toLowerCase()))
                .toList();

            default -> List.of();
        };
    }

}
