package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.shrinkingBorder;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.appliance.Appliance;
import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import eu.mhsl.craftattack.spawn.core.config.Configuration;
import eu.mhsl.craftattack.spawn.common.appliances.metaGameplay.settings.Settings;
import org.bukkit.Bukkit;
import org.bukkit.World;
import org.bukkit.WorldBorder;
import org.bukkit.event.Listener;
import org.jetbrains.annotations.NotNull;

import java.time.LocalTime;
import java.util.*;

public class ShrinkingBorder extends Appliance {
    private final String shrinkPerDayKey = "shrinkPerDay";
    private final String shrinkTimeKey = "shrinkTime";
    private final String minimumSizeKey = "minimumSize";
    public int shrinkPerDay;
    public LocalTime shrinkTime;
    public int minimumSize;
    private final Timer timer = new Timer();
    private final Calendar calendar = Calendar.getInstance(TimeZone.getDefault());

    private final WorldBorder overworldBorder = Bukkit.getWorlds().stream()
        .filter(world -> world.getEnvironment().equals(World.Environment.NORMAL))
        .findFirst().orElseThrow().getWorldBorder();
    private final WorldBorder netherBorder = Bukkit.getWorlds().stream()
        .filter(world -> world.getEnvironment().equals(World.Environment.NETHER))
        .findFirst().orElseThrow().getWorldBorder();

    private class ShrinkBorderTask extends TimerTask {
        @Override
        public void run() {
            Bukkit.getScheduler().runTask(Main.instance(), ShrinkingBorder.this::shrinkBorder);
        }
    }

    @Override
    public void onEnable() {
        Settings.instance().declareSetting(ShrinkingBorderSetting.class);
    }

    public ShrinkingBorder() {
        super("shrinkingBorder");
        this.shrinkPerDay = this.localConfig().getInt(this.shrinkPerDayKey, 10);
        this.shrinkTime = LocalTime.parse(this.localConfig().getString(this.shrinkTimeKey, LocalTime.of(3, 0, 0).toString()));
        this.minimumSize = this.localConfig().getInt(this.minimumSizeKey, 10);

        this.calendar.set(Calendar.HOUR_OF_DAY, this.shrinkTime.getHour());
        this.calendar.set(Calendar.MINUTE, this.shrinkTime.getMinute());
        this.calendar.set(Calendar.SECOND, this.shrinkTime.getSecond());

        if(this.calendar.getTime().before(new Date(System.currentTimeMillis()))) {
            this.calendar.add(Calendar.DAY_OF_MONTH, 1);
        }

        this.timer.schedule(new ShrinkBorderTask(), this.calendar.getTime());
    }

    private void shrinkBorder() {
        this.overworldBorder.setSize(Math.max(this.overworldBorder.getSize()-this.shrinkPerDay, this.minimumSize));
        this.netherBorder.setSize(Math.max(this.netherBorder.getSize()-this.shrinkPerDay, this.minimumSize));

        this.calendar.add(Calendar.DAY_OF_MONTH, 1);
        this.timer.schedule(new ShrinkBorderTask(), this.calendar.getTime());
    }

    public void changeOption(ShrinkingBorderCommand.Argument option, int value) {
        switch (option) {
            case MINIMUM_SIZE -> {
                this.minimumSize = value;
                this.localConfig().set(this.minimumSizeKey, value);
            }
            case SHRINK_PER_DAY -> {
                this.shrinkPerDay = value;
                this.localConfig().set(this.shrinkPerDayKey, value);
            }
        }
        Configuration.saveChanges();
    }

    public int getOption(ShrinkingBorderCommand.Argument option) {
        switch (option) {
            case MINIMUM_SIZE -> {
                return this.minimumSize;
            }
            case SHRINK_PER_DAY -> {
                return this.shrinkPerDay;
            }
            default -> {
                return 0;
            }
        }
    }

    @Override
    protected @NotNull List<ApplianceCommand<?>> commands() {
        return List.of(new ShrinkingBorderCommand());
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new ShrinkingBorderListener());
    }
}
