package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.shrinkingBorder;

import eu.mhsl.craftattack.spawn.core.appliance.ApplianceCommand;
import org.bukkit.command.Command;
import org.bukkit.command.CommandSender;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.Arrays;
import java.util.List;

public class ShrinkingBorderCommand extends ApplianceCommand<ShrinkingBorder> {
     public enum Argument {
        SHRINK_PER_DAY("shrinkPerDay"),
        MINIMUM_SIZE("minimumSize");

        public final String name;

        Argument(String friendlyName) {
            this.name = friendlyName;
        }

        static Argument fromString(String argument) {
            return Arrays.stream(values()).filter(argument1 -> argument1.name.equals(argument)).findFirst().orElseThrow();
        }
    }

    public ShrinkingBorderCommand() {
        super("shrinkingBorder");
    }

    @Override
    protected void execute(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if(args.length < 1 || args.length > 2) throw new Error("shrinkingBorder command needs one or two arguments");
        if(Arrays.stream(Argument.values()).noneMatch(argument -> argument.name.equals(args[0]))) throw new Error(String.format("argument %s does not exist", args[0]));
        Argument option = Argument.fromString(args[0]);

        switch (args.length) {
            case 1:
                sender.sendMessage(String.format("%s is currently set to %d", args[0], this.getAppliance().getOption(option)));
                break;
            case 2:
                try {
                    int value = Integer.parseInt(args[1]);
                    this.getAppliance().changeOption(option, value);
                    sender.sendMessage(String.format("%s is now set to %d", args[0], this.getAppliance().getOption(option)));
                } catch (NumberFormatException e) {
                    throw new Error("second argument has to be an integer");
                }
                break;
            default:
                throw new Error("shrinkingBorder command needs one or two arguments");
        }
    }

    @Override
    public @Nullable List<String> onTabComplete(@NotNull CommandSender sender, @NotNull Command command, @NotNull String label, @NotNull String[] args) {
        if(args.length > 1) return List.of();
        return Arrays.stream(Argument.values()).map(argument -> argument.name).toList();
    }
}
