package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.teams;

import eu.mhsl.craftattack.spawn.core.Main;
import eu.mhsl.craftattack.spawn.core.util.text.DisconnectInfo;
import eu.mhsl.craftattack.spawn.varo.appliances.internal.teamTasks.TeamTasks;
import eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.fightDetector.FightDetector;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.OfflinePlayer;
import org.bukkit.entity.Player;
import org.jetbrains.annotations.Nullable;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

public class VaroTeam {
    public static class Member {
        public final OfflinePlayer player;
        public boolean isDead;

        public Member(UUID playerId, boolean isDead) {
            this.player = Bukkit.getOfflinePlayer(playerId);
            this.isDead = isDead;
        }
    }

    public String name;
    public String color;
    public List<Member> members;

    public VaroTeam(List<Member> members, String name, String color) {
        this.members = new ArrayList<>(members);
        this.name = name;
        this.color = color;
    }

    public boolean isTeamDead() {
        return this.members.stream().allMatch(member -> member.isDead);
    }

    public boolean hasMember(UUID uuid) {
        return this.members.stream()
            .anyMatch(member -> member.player.getUniqueId().equals(uuid));
    }

    public @Nullable Member getMemberById(UUID uuid) {
        return this.members.stream()
            .filter(member -> member.player.getUniqueId().equals(uuid))
            .findFirst()
            .orElse(null);
    }

    public void kickTeam() {
        this.kickTeam(new DisconnectInfo(
            "Unbekannter Fehler",
            "Verbindung wurde aufgrund eines unbekannten Grundes getrennt",
            "Falls du denkst, dass dies ein Fehler ist, melde dich bei einem Admin!",
            UUID.randomUUID()
        ));
    }

    public List<Player> getOnlinePlayers() {
        return this.members.stream()
            .map(member -> Bukkit.getPlayer(member.player.getUniqueId()))
            .filter(Objects::nonNull)
            .toList();
    }

    public void kickTeam(DisconnectInfo disconnectInfo) {
        this.getOnlinePlayers()
            .forEach(player -> player.kick(disconnectInfo.getComponent()));
        Main.instance().getAppliance(TeamTasks.class).cancelTeamTasks(this);
    }

    public void timeOverKick() {
        boolean isInFight = Main.instance().getAppliance(FightDetector.class).isInFight(this);
        if(isInFight) {
            Main.logger().info(String.format("Cannot kick Team %s because it is in a fight!", this.name));
            this.getOnlinePlayers()
                    .forEach(player -> player.sendMessage("Du befindest dich in einer Kampfhandlung oder unmittelbar"));
            Bukkit.getScheduler().runTaskLater(
                Main.instance(),
                this::timeOverKick,
                Ticks.TICKS_PER_SECOND * 15
            );
            return;
        }

        Main.logger().info(String.format("Kicking Team %s because time is up!", this.name));

        DisconnectInfo timeOverInfo = new DisconnectInfo(
            "Die Zeit ist abgelaufen!",
            "Deine Spielzeit ist vorüber. Falls dir noch weitere Zeit zusteht kannst du jetzt eine Pause machen und anschließend erneut beitreten.",
            "Falls du Fragen hast, melde dich bitte bei einem Admin!",
            UUID.nameUUIDFromBytes("".getBytes())
        );

        this.kickTeam(timeOverInfo);
    }
}
